@@ -1,7 +1,7 @@
 
 -- |
 -- Module     : Simulation.Aivika.Event
--- Copyright  : Copyright (c) 2009-2013, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2015, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
@@ -10,6 +10,12 @@
 -- The module defines the 'Event' monad which is very similar to the 'Dynamics'
 -- monad but only now the computation is strongly synchronized with the event queue.
 --
+-- The @Dynamics@ computation is defined in all time points simultaneously, while
+-- the @Event@ computation can be described in every time point differently and can change
+-- in discrete steps. Therefore, the former is destined for differential and difference
+-- equations of System Dynamics, while the latter is destined for discrete event simulation,
+-- being its core actually.
+--
 module Simulation.Aivika.Event
        (-- * Event Monad
         Event,
@@ -39,6 +45,8 @@ module Simulation.Aivika.Event
         memoEvent,
         memoEventInTime,
         -- * Disposable
-        DisposableEvent(..)) where
+        DisposableEvent(..),
+        -- * Debugging
+        traceEvent) where
 
 import Simulation.Aivika.Internal.Event
