@@ -1,27 +1,38 @@
 
 -- |
 -- Module     : Simulation.Aivika.Internal.Cont
--- Copyright  : Copyright (c) 2009-2013, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2015, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
--- Tested with: GHC 7.6.3
+-- Tested with: GHC 7.8.3
 --
--- The 'Cont' monad is a variation of the standard Cont monad 
+-- This is an internal implementation module that should never be used directly.
+--
+-- The 'Cont' monad is a variation of the standard @Cont@ monad 
 -- and F# async workflow, where the result of applying 
 -- the continuations is the 'Event' computation.
 --
 module Simulation.Aivika.Internal.Cont
        (ContCancellation(..),
-        ContCancellationSource,
+        ContId,
+        ContEvent(..),
         Cont(..),
         ContParams,
-        newContCancellationSource,
+        FrozenCont,
+        newContId,
+        contSignal,
         contCancellationInitiated,
         contCancellationInitiate,
         contCancellationInitiating,
+        contCancellationActivated,
         contCancellationBind,
         contCancellationConnect,
+        contPreemptionBegun,
+        contPreemptionBegin,
+        contPreemptionBeginning,
+        contPreemptionEnd,
+        contPreemptionEnding,
         invokeCont,
         runCont,
         rerunCont,
@@ -33,28 +44,33 @@ module Simulation.Aivika.Internal.Cont
         throwCont,
         resumeCont,
         resumeECont,
+        reenterCont,
+        freezeCont,
+        freezeContReentering,
+        unfreezeCont,
+        substituteCont,
         contCanceled,
-        contFreeze,
-        contAwait) where
+        contAwait,
+        traceCont) where
 
 import Data.IORef
 import Data.Array
 import Data.Array.IO.Safe
 import Data.Monoid
 
-import qualified Control.Exception as C
-import Control.Exception (IOException, throw)
-
+import Control.Exception
 import Control.Monad
 import Control.Monad.Trans
 import Control.Applicative
 
+import Debug.Trace
+
 import Simulation.Aivika.Internal.Specs
 import Simulation.Aivika.Internal.Parameter
 import Simulation.Aivika.Internal.Simulation
 import Simulation.Aivika.Internal.Dynamics
 import Simulation.Aivika.Internal.Event
-import Simulation.Aivika.Internal.Signal
+import Simulation.Aivika.Signal
 
 -- | It defines how the parent and child computations should be cancelled.
 data ContCancellation = CancelTogether
@@ -66,47 +82,66 @@ data ContCancellation = CancelTogether
                       | CancelInIsolation
                         -- ^ Cancel the computations in isolation.
 
--- | It manages the cancellation process.
-data ContCancellationSource =
-  ContCancellationSource { contCancellationInitiatedRef :: IORef Bool,
-                           contCancellationActivatedRef :: IORef Bool,
-                           contCancellationInitiatingSource :: SignalSource ()
-                         }
+-- | It identifies the 'Cont' computation.
+data ContId =
+  ContId { contCancellationInitiatedRef :: IORef Bool,
+           contCancellationActivatedRef :: IORef Bool,
+           contPreemptionCountRef :: IORef Int,
+           contSignalSource :: SignalSource ContEvent
+         }
 
--- | Create the cancellation source.
-newContCancellationSource :: Simulation ContCancellationSource
-newContCancellationSource =
+instance Eq ContId where
+  x == y = contCancellationInitiatedRef x == contCancellationInitiatedRef y  -- unique references
+
+-- | The event that occurs within the 'Cont' computation.
+data ContEvent = ContCancellationInitiating
+                 -- ^ Cancel the computation.
+               | ContPreemptionBeginning
+                 -- ^ Preempt the computation.
+               | ContPreemptionEnding
+                 -- ^ Proceed with the computation after if was preempted.
+               deriving (Eq, Ord, Show)
+
+-- | Create a computation identifier.
+newContId :: Simulation ContId
+newContId =
   Simulation $ \r ->
   do r1 <- newIORef False
      r2 <- newIORef False
+     r3 <- newIORef 0
      s  <- invokeSimulation r newSignalSource
-     return ContCancellationSource { contCancellationInitiatedRef = r1,
-                                     contCancellationActivatedRef = r2,
-                                     contCancellationInitiatingSource = s
-                                   }
+     return ContId { contCancellationInitiatedRef = r1,
+                     contCancellationActivatedRef = r2,
+                     contPreemptionCountRef = r3,
+                     contSignalSource = s
+                   }
+
+-- | Signal when the computation state changes.
+contSignal :: ContId -> Signal ContEvent
+contSignal = publishSignal . contSignalSource
 
 -- | Signal when the cancellation is intiating.
-contCancellationInitiating :: ContCancellationSource -> Signal ()
+contCancellationInitiating :: ContId -> Signal ()
 contCancellationInitiating =
-  publishSignal . contCancellationInitiatingSource
+  filterSignal_ (ContCancellationInitiating ==) . contSignal
 
 -- | Whether the cancellation was initiated.
-contCancellationInitiated :: ContCancellationSource -> Event Bool
+contCancellationInitiated :: ContId -> Event Bool
 contCancellationInitiated x =
   Event $ \p -> readIORef (contCancellationInitiatedRef x)
 
 -- | Whether the cancellation was activated.
-contCancellationActivated :: ContCancellationSource -> IO Bool
+contCancellationActivated :: ContId -> IO Bool
 contCancellationActivated =
   readIORef . contCancellationActivatedRef
 
 -- | Deactivate the cancellation.
-contCancellationDeactivate :: ContCancellationSource -> IO ()
+contCancellationDeactivate :: ContId -> IO ()
 contCancellationDeactivate x =
   writeIORef (contCancellationActivatedRef x) False
 
 -- | If the main computation is cancelled then all the nested ones will be cancelled too.
-contCancellationBind :: ContCancellationSource -> [ContCancellationSource] -> Event DisposableEvent
+contCancellationBind :: ContId -> [ContId] -> Event DisposableEvent
 contCancellationBind x ys =
   Event $ \p ->
   do hs1 <- forM ys $ \y ->
@@ -120,11 +155,11 @@ contCancellationBind x ys =
      return $ mconcat hs1 <> mconcat hs2
 
 -- | Connect the parent computation to the child one.
-contCancellationConnect :: ContCancellationSource
+contCancellationConnect :: ContId
                            -- ^ the parent
                            -> ContCancellation
                            -- ^ how to connect
-                           -> ContCancellationSource
+                           -> ContId
                            -- ^ the child
                            -> Event DisposableEvent
                            -- ^ computation of the disposable handler
@@ -151,16 +186,60 @@ contCancellationConnect parent cancellation child =
      return $ h1 <> h2
 
 -- | Initiate the cancellation.
-contCancellationInitiate :: ContCancellationSource -> Event ()
+contCancellationInitiate :: ContId -> Event ()
 contCancellationInitiate x =
   Event $ \p ->
   do f <- readIORef (contCancellationInitiatedRef x)
      unless f $
        do writeIORef (contCancellationInitiatedRef x) True
           writeIORef (contCancellationActivatedRef x) True
-          invokeEvent p $ triggerSignal (contCancellationInitiatingSource x) ()
+          invokeEvent p $
+            triggerSignal (contSignalSource x) ContCancellationInitiating
 
--- | The 'Cont' type is similar to the standard Cont monad 
+-- | Preempt the computation.
+contPreemptionBegin :: ContId -> Event ()
+contPreemptionBegin x =
+  Event $ \p ->
+  do f <- readIORef (contCancellationInitiatedRef x)
+     unless f $
+       do n <- readIORef (contPreemptionCountRef x)
+          let n' = n + 1
+          n' `seq` writeIORef (contPreemptionCountRef x) n'
+          when (n == 0) $
+            invokeEvent p $
+            triggerSignal (contSignalSource x) ContPreemptionBeginning
+
+-- | Proceed with the computation after it was preempted earlier.
+contPreemptionEnd :: ContId -> Event ()
+contPreemptionEnd x =
+  Event $ \p ->
+  do f <- readIORef (contCancellationInitiatedRef x)
+     unless f $
+       do n <- readIORef (contPreemptionCountRef x)
+          let n' = n - 1
+          n' `seq` writeIORef (contPreemptionCountRef x) n'
+          when (n' == 0) $
+            invokeEvent p $
+            triggerSignal (contSignalSource x) ContPreemptionEnding
+
+-- | Signal when the computation is preempted.
+contPreemptionBeginning :: ContId -> Signal ()
+contPreemptionBeginning =
+  filterSignal_ (ContPreemptionBeginning ==) . contSignal
+
+-- | Signal when the computation is proceeded after it was preempted before.
+contPreemptionEnding :: ContId -> Signal ()
+contPreemptionEnding =
+  filterSignal_ (ContPreemptionEnding ==) . contSignal
+
+-- | Whether the computation was preemtped.
+contPreemptionBegun :: ContId -> Event Bool
+contPreemptionBegun x =
+  Event $ \p ->
+  do n <- readIORef (contPreemptionCountRef x)
+     return (n > 0)
+
+-- | The 'Cont' type is similar to the standard @Cont@ monad 
 -- and F# async workflow but only the result of applying
 -- the continuations return the 'Event' computation.
 newtype Cont a = Cont (ContParams a -> Event ())
@@ -172,9 +251,9 @@ data ContParams a =
 
 -- | The auxiliary continuation parameters.
 data ContParamsAux =
-  ContParamsAux { contECont :: IOException -> Event (),
+  ContParamsAux { contECont :: SomeException -> Event (),
                   contCCont :: () -> Event (),
-                  contCancelSource :: ContCancellationSource,
+                  contId :: ContId,
                   contCancelFlag :: IO Bool,
                   contCatchFlag  :: Bool }
 
@@ -213,7 +292,7 @@ invokeCont p (Cont m) = m p
 cancelCont :: Point -> ContParams a -> IO ()
 {-# NOINLINE cancelCont #-}
 cancelCont p c =
-  do contCancellationDeactivate (contCancelSource $ contAux c)
+  do contCancellationDeactivate (contId $ contAux c)
      invokeEvent p $ (contCCont $ contAux c) ()
 
 returnC :: a -> Cont a
@@ -226,22 +305,10 @@ returnC a =
        then cancelCont p c
        else invokeEvent p $ contCont c a
                           
--- bindC :: Cont a -> (a -> Cont b) -> Cont b
--- {-# INLINE bindC #-}
--- bindC m k = 
---   Cont $ \c -> 
---   if (contCatchFlag . contAux $ c) 
---   then bindWithCatch m k c
---   else bindWithoutCatch m k c
-  
 bindC :: Cont a -> (a -> Cont b) -> Cont b
 {-# INLINE bindC #-}
-bindC m k = 
-  Cont $ bindWithoutCatch m k  -- Another version is not tail recursive!
-  
-bindWithoutCatch :: Cont a -> (a -> Cont b) -> ContParams b -> Event ()
-{-# INLINE bindWithoutCatch #-}
-bindWithoutCatch (Cont m) k c = 
+bindC (Cont m) k =
+  Cont $ \c ->
   Event $ \p ->
   do z <- contCanceled c
      if z 
@@ -250,67 +317,38 @@ bindWithoutCatch (Cont m) k c =
             let cont a = invokeCont c (k a)
             in c { contCont = cont }
 
--- -- It is not tail recursive!
--- bindWithCatch :: Cont a -> (a -> Cont b) -> ContParams b -> Event ()
--- {-# NOINLINE bindWithCatch #-}
--- bindWithCatch (Cont m) k c = 
---   Event $ \p ->
---   do z <- contCanceled c
---      if z 
---        then cancelCont p c
---        else invokeEvent p $ m $ 
---             let cont a = catchEvent 
---                          (invokeCont c (k a))
---                          (contECont $ contAux c)
---             in c { contCont = cont }
-
--- Like "bindWithoutCatch (return a) k"
-callWithoutCatch :: (a -> Cont b) -> a -> ContParams b -> Event ()
-callWithoutCatch k a c =
+-- | Like @return a >>= k@.
+callCont :: (a -> Cont b) -> a -> ContParams b -> Event ()
+callCont k a c =
   Event $ \p ->
   do z <- contCanceled c
      if z 
        then cancelCont p c
        else invokeEvent p $ invokeCont c (k a)
 
--- -- Like "bindWithCatch (return a) k" but it is not tail recursive!
--- callWithCatch :: (a -> Cont b) -> a -> ContParams b -> Event ()
--- callWithCatch k a c =
---   Event $ \p ->
---   do z <- contCanceled c
---      if z 
---        then cancelCont p c
---        else invokeEvent p $ catchEvent 
---             (invokeCont c (k a))
---             (contECont $ contAux c)
-
 -- | Exception handling within 'Cont' computations.
-catchCont :: Cont a -> (IOException -> Cont a) -> Cont a
-catchCont m h = 
-  Cont $ \c ->
-  catchWithCatch m h (c { contAux = (contAux c) { contCatchFlag = True } })
-  
-catchWithCatch :: Cont a -> (IOException -> Cont a) -> ContParams a -> Event ()
-catchWithCatch (Cont m) h c =
-  Event $ \p -> 
-  do z <- contCanceled c
+catchCont :: Exception e => Cont a -> (e -> Cont a) -> Cont a
+catchCont (Cont m) h = 
+  Cont $ \c0 ->
+  Event $ \p ->
+  do let c = c0 { contAux = (contAux c0) { contCatchFlag = True } }
+     z <- contCanceled c
      if z 
        then cancelCont p c
        else invokeEvent p $ m $
-            -- let econt e = callWithCatch h e c   -- not tail recursive!
-            let econt e = callWithoutCatch h e c
+            let econt e0 =
+                  case fromException e0 of
+                    Just e  -> callCont h e c
+                    Nothing -> (contECont . contAux $ c) e0
             in c { contAux = (contAux c) { contECont = econt } }
                
 -- | A computation with finalization part.
 finallyCont :: Cont a -> Cont b -> Cont a
-finallyCont m m' = 
-  Cont $ \c -> 
-  finallyWithCatch m m' (c { contAux = (contAux c) { contCatchFlag = True } })
-  
-finallyWithCatch :: Cont a -> Cont b -> ContParams a -> Event ()               
-finallyWithCatch (Cont m) (Cont m') c =
+finallyCont (Cont m) (Cont m') = 
+  Cont $ \c0 ->
   Event $ \p ->
-  do z <- contCanceled c
+  do let c = c0 { contAux = (contAux c0) { contCatchFlag = True } }
+     z <- contCanceled c
      if z 
        then cancelCont p c
        else invokeEvent p $ m $
@@ -336,12 +374,14 @@ finallyWithCatch (Cont m) (Cont m') c =
                                             contCCont = ccont } }
 
 -- | Throw the exception with the further exception handling.
--- By some reasons, the standard 'throw' function per se is not handled 
--- properly within 'Cont' computations, altough it will be still handled 
--- if it will be hidden under the 'liftIO' function. The problem arises 
--- namely with the @throw@ function, not 'IO' computations.
+--
+-- By some reason, an exception raised with help of the standard 'throw' function
+-- is not handled properly within 'Cont' computation, altough it will be still handled 
+-- if it will be wrapped in the 'IO' monad. Therefore, you should use specialised
+-- functions like the stated one that use the 'throw' function but within the 'IO' computation,
+-- which allows already handling the exception.
 throwCont :: IOException -> Cont a
-throwCont e = liftIO $ throw e
+throwCont = liftIO . throw
 
 -- | Run the 'Cont' computation with the specified cancelation source 
 -- and flag indicating whether to catch exceptions from the beginning.
@@ -349,22 +389,22 @@ runCont :: Cont a
            -- ^ the computation to run
            -> (a -> Event ())
            -- ^ the main branch 
-           -> (IOError -> Event ())
+           -> (SomeException -> Event ())
            -- ^ the branch for handing exceptions
            -> (() -> Event ())
            -- ^ the branch for cancellation
-           -> ContCancellationSource
-           -- ^ the cancellation source
+           -> ContId
+           -- ^ the computation identifier
            -> Bool
            -- ^ whether to support the exception handling from the beginning
            -> Event ()
-runCont (Cont m) cont econt ccont cancelSource catchFlag = 
+runCont (Cont m) cont econt ccont cid catchFlag = 
   m ContParams { contCont = cont,
                  contAux  = 
                    ContParamsAux { contECont = econt,
                                    contCCont = ccont,
-                                   contCancelSource = cancelSource,
-                                   contCancelFlag = contCancellationActivated cancelSource, 
+                                   contId = cid,
+                                   contCancelFlag = contCancellationActivated cid, 
                                    contCatchFlag  = catchFlag } }
 
 -- | Lift the 'Parameter' computation.
@@ -429,7 +469,7 @@ liftIOWithCatch m p c =
        then cancelCont p c
        else do aref <- newIORef undefined
                eref <- newIORef Nothing
-               C.catch (m >>= writeIORef aref) 
+               catch (m >>= writeIORef aref) 
                  (writeIORef eref . Just)
                e <- readIORef eref
                case e of
@@ -452,7 +492,7 @@ resumeCont c a =
        else invokeEvent p $ contCont c a
 
 -- | Resume the exception handling by the specified parameters.
-resumeECont :: ContParams a -> IOException -> Event ()
+resumeECont :: ContParams a -> SomeException -> Event ()
 {-# INLINE resumeECont #-}
 resumeECont c e = 
   Event $ \p ->
@@ -476,10 +516,10 @@ contCanceled c = contCancelFlag $ contAux c
 -- Here word @parallel@ literally means that the computations are
 -- actually executed on a single operating system thread but
 -- they are processed simultaneously by the event queue.
-contParallel :: [(Cont a, ContCancellationSource)]
+contParallel :: [(Cont a, ContId)]
                 -- ^ the list of:
                 -- the nested computation,
-                -- the cancellation source
+                -- the computation identifier
                 -> Cont [a]
 contParallel xs =
   Cont $ \c ->
@@ -490,7 +530,7 @@ contParallel xs =
               counter   <- newIORef 0
               catchRef  <- newIORef Nothing
               hs <- invokeEvent p $
-                    contCancellationBind (contCancelSource $ contAux c) $
+                    contCancellationBind (contId $ contAux c) $
                     map snd xs
               let propagate =
                     Event $ \p ->
@@ -525,9 +565,9 @@ contParallel xs =
                     do modifyIORef counter (+ 1)
                        -- the main computation was automatically canceled
                        invokeEvent p propagate
-              forM_ (zip [1..n] xs) $ \(i, (x, cancelSource)) ->
+              forM_ (zip [1..n] xs) $ \(i, (x, cid)) ->
                 invokeEvent p $
-                runCont x (cont i) econt ccont cancelSource (contCatchFlag $ contAux c)
+                runCont x (cont i) econt ccont cid (contCatchFlag $ contAux c)
      z <- contCanceled c
      if z
        then cancelCont p c
@@ -538,10 +578,10 @@ contParallel xs =
 -- | A partial case of 'contParallel' when we are not interested in
 -- the results but we are interested in the actions to be peformed by
 -- the nested computations.
-contParallel_ :: [(Cont a, ContCancellationSource)]
+contParallel_ :: [(Cont a, ContId)]
                  -- ^ the list of:
                  -- the nested computation,
-                 -- the cancellation source
+                 -- the computation identifier
                  -> Cont ()
 contParallel_ xs =
   Cont $ \c ->
@@ -551,7 +591,7 @@ contParallel_ xs =
            do counter   <- newIORef 0
               catchRef  <- newIORef Nothing
               hs <- invokeEvent p $
-                    contCancellationBind (contCancelSource $ contAux c) $
+                    contCancellationBind (contId $ contAux c) $
                     map snd xs
               let propagate =
                     Event $ \p ->
@@ -585,9 +625,9 @@ contParallel_ xs =
                     do modifyIORef counter (+ 1)
                        -- the main computation was automatically canceled
                        invokeEvent p propagate
-              forM_ (zip [1..n] xs) $ \(i, (x, cancelSource)) ->
+              forM_ (zip [1..n] xs) $ \(i, (x, cid)) ->
                 invokeEvent p $
-                runCont x (cont i) econt ccont cancelSource (contCatchFlag $ contAux c)
+                runCont x (cont i) econt ccont cid (contCatchFlag $ contAux c)
      z <- contCanceled c
      if z
        then cancelCont p c
@@ -595,14 +635,14 @@ contParallel_ xs =
             then invokeEvent p $ contCont c ()
             else worker
 
--- | Rerun the 'Cont' computation with the specified cancellation source.
-rerunCont :: Cont a -> ContCancellationSource -> Cont a
-rerunCont x cancelSource =
+-- | Rerun the 'Cont' computation with the specified identifier.
+rerunCont :: Cont a -> ContId -> Cont a
+rerunCont x cid =
   Cont $ \c ->
   Event $ \p ->
   do let worker =
            do hs <- invokeEvent p $
-                    contCancellationBind (contCancelSource $ contAux c) [cancelSource]
+                    contCancellationBind (contId $ contAux c) [cid]
               let cont a  =
                     Event $ \p ->
                     do invokeEvent p $ disposeEvent hs  -- unbind the cancellation source
@@ -616,21 +656,21 @@ rerunCont x cancelSource =
                     do invokeEvent p $ disposeEvent hs  -- unbind the cancellation source
                        cancelCont p c
               invokeEvent p $
-                runCont x cont econt ccont cancelSource (contCatchFlag $ contAux c)
+                runCont x cont econt ccont cid (contCatchFlag $ contAux c)
      z <- contCanceled c
      if z
        then cancelCont p c
        else worker
 
--- | Run the 'Cont' computation in parallel but connect the cancellation sources.
-spawnCont :: ContCancellation -> Cont () -> ContCancellationSource -> Cont ()
-spawnCont cancellation x cancelSource =
+-- | Run the 'Cont' computation in parallel but connect the computations.
+spawnCont :: ContCancellation -> Cont () -> ContId -> Cont ()
+spawnCont cancellation x cid =
   Cont $ \c ->
   Event $ \p ->
   do let worker =
            do hs <- invokeEvent p $
                     contCancellationConnect
-                    (contCancelSource $ contAux c) cancellation cancelSource
+                    (contId $ contAux c) cancellation cid
               let cont a  =
                     Event $ \p ->
                     do invokeEvent p $ disposeEvent hs  -- unbind the cancellation source
@@ -645,7 +685,7 @@ spawnCont cancellation x cancelSource =
                        -- do nothing and it will finish the computation
               invokeEvent p $
                 enqueueEvent (pointTime p) $
-                runCont x cont econt ccont cancelSource False
+                runCont x cont econt ccont cid False
               invokeEvent p $
                 resumeCont c ()
      z <- contCanceled c
@@ -653,21 +693,26 @@ spawnCont cancellation x cancelSource =
        then cancelCont p c
        else worker
 
+-- | Represents a temporarily frozen computation.
+newtype FrozenCont a =
+  FrozenCont { unfreezeCont :: Event (Maybe (ContParams a))
+               -- ^ Unfreeze the computation.
+             }
+
 -- | Freeze the computation parameters temporarily.
-contFreeze :: ContParams a -> Event (Event (Maybe (ContParams a)))
-contFreeze c =
+freezeCont :: ContParams a -> Event (FrozenCont a)
+freezeCont c =
   Event $ \p ->
   do rh <- newIORef Nothing
      rc <- newIORef $ Just c
      h <- invokeEvent p $
           handleSignal (contCancellationInitiating $
-                        contCancelSource $
-                        contAux c) $ \a ->
+                        contId $ contAux c) $ \e ->
           Event $ \p ->
           do h <- readIORef rh
              case h of
                Nothing ->
-                 error "The handler was lost: contFreeze."
+                 error "The handler was lost: freezeCont."
                Just h ->
                  do invokeEvent p $ disposeEvent h
                     c <- readIORef rc
@@ -682,18 +727,115 @@ contFreeze c =
                                 when z $ cancelCont p c
      writeIORef rh (Just h)
      return $
+       FrozenCont $
        Event $ \p ->
        do invokeEvent p $ disposeEvent h
           c <- readIORef rc
           writeIORef rc Nothing
           return c
-     
+
+-- | Freeze the computation parameters specifying what should be done when reentering the computation.
+freezeContReentering :: ContParams a -> a -> Event () -> Event (FrozenCont a)
+freezeContReentering c a m =
+  Event $ \p ->
+  do rh <- newIORef Nothing
+     rc <- newIORef $ Just c
+     h <- invokeEvent p $
+          handleSignal (contCancellationInitiating $
+                        contId $ contAux c) $ \e ->
+          Event $ \p ->
+          do h <- readIORef rh
+             case h of
+               Nothing ->
+                 error "The handler was lost: freezeContReentering."
+               Just h ->
+                 do invokeEvent p $ disposeEvent h
+                    c <- readIORef rc
+                    case c of
+                      Nothing -> return ()
+                      Just c  ->
+                        do writeIORef rc Nothing
+                           invokeEvent p $
+                             enqueueEvent (pointTime p) $
+                             Event $ \p ->
+                             do z <- contCanceled c
+                                when z $ cancelCont p c
+     writeIORef rh (Just h)
+     return $
+       FrozenCont $
+       Event $ \p ->
+       do invokeEvent p $ disposeEvent h
+          c <- readIORef rc
+          writeIORef rc Nothing
+          case c of
+            Nothing -> return Nothing
+            z @ (Just c) ->
+              do f <- invokeEvent p $
+                      contPreemptionBegun $
+                      contId $ contAux c
+                 if not f
+                   then return z
+                   else do let c = c { contCont = \a -> m }
+                           invokeEvent p $ sleepCont c a
+                           return Nothing
+
+-- | Reenter the computation parameters when needed.
+reenterCont :: ContParams a -> a -> Event ()
+{-# INLINE reenterCont #-}
+reenterCont c a =
+  Event $ \p ->
+  do f <- invokeEvent p $
+          contPreemptionBegun $
+          contId $ contAux c
+     if not f
+       then invokeEvent p $
+            enqueueEvent (pointTime p) $
+            resumeCont c a
+       else invokeEvent p $
+            sleepCont c a
+
+-- | Sleep until the preempted computation will be reentered.
+sleepCont :: ContParams a -> a -> Event ()
+{-# NOINLINE sleepCont #-}
+sleepCont c a =
+  Event $ \p ->
+  do rh <- newIORef Nothing
+     h  <- invokeEvent p $
+           handleSignal (contSignal $
+                         contId $ contAux c) $ \e ->
+           Event $ \p ->
+           do h <- readIORef rh
+              case h of
+                Nothing ->
+                  error "The handler was lost: sleepCont."
+                Just h ->
+                  do invokeEvent p $ disposeEvent h
+                     case e of
+                       ContCancellationInitiating ->
+                         invokeEvent p $
+                         enqueueEvent (pointTime p) $
+                         Event $ \p ->
+                         do z <- contCanceled c
+                            when z $ cancelCont p c
+                       ContPreemptionEnding ->
+                         invokeEvent p $
+                         enqueueEvent (pointTime p) $
+                         resumeCont c a
+                       ContPreemptionBeginning ->
+                         error "The computation was already preempted: sleepCont."
+     writeIORef rh (Just h)
+
+-- | Substitute the continuation.
+substituteCont :: ContParams a -> (a -> Event ()) -> ContParams a
+{-# INLINE substituteCont #-}
+substituteCont c m = c { contCont = m }
+
 -- | Await the signal.
 contAwait :: Signal a -> Cont a
 contAwait signal =
   Cont $ \c ->
   Event $ \p ->
-  do c <- invokeEvent p $ contFreeze c
+  do c <- invokeEvent p $ freezeCont c
      r <- newIORef Nothing
      h <- invokeEvent p $
           handleSignal signal $ 
@@ -704,9 +846,20 @@ contAwait signal =
                              error "The signal was lost: contAwait."
                            Just x ->
                              do invokeEvent p $ disposeEvent x
-                                c <- invokeEvent p c
+                                c <- invokeEvent p $ unfreezeCont c
                                 case c of
                                   Nothing -> return ()
                                   Just c  ->
-                                    invokeEvent p $ resumeCont c a
+                                    invokeEvent p $ reenterCont c a
      writeIORef r $ Just h          
+
+-- | Show the debug message with the current simulation time.
+traceCont :: String -> Cont a -> Cont a
+traceCont message (Cont m) =
+  Cont $ \c ->
+  Event $ \p ->
+  do z <- contCanceled c
+     if z
+       then cancelCont p c
+       else trace ("t = " ++ show (pointTime p) ++ ": " ++ message) $
+            invokeEvent p $ m c
