@@ -1,11 +1,13 @@
 
 -- |
 -- Module     : Simulation.Aivika.Internal.Process
--- Copyright  : Copyright (c) 2009-2013, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2015, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
--- Tested with: GHC 7.6.3
+-- Tested with: GHC 7.8.3
+--
+-- This is an internal implementation module that should never be used directly.
 --
 -- A value in the 'Process' monad represents a discontinuous process that 
 -- can suspend in any simulation time point and then resume later in the same 
@@ -17,6 +19,9 @@
 --
 -- A value of the 'ProcessId' type is just an identifier of such a process.
 --
+-- The characteristic property of the @Process@ type is function 'holdProcess'
+-- that suspends the current process for the specified time interval.
+--
 module Simulation.Aivika.Internal.Process
        (-- * Process Monad
         ProcessId,
@@ -33,6 +38,8 @@ module Simulation.Aivika.Internal.Process
         -- * Spawning Processes
         spawnProcess,
         spawnProcessUsingId,
+        spawnProcessWith,
+        spawnProcessUsingIdWith,
         -- * Enqueuing Process
         enqueueProcess,
         enqueueProcessUsingId,
@@ -54,6 +61,11 @@ module Simulation.Aivika.Internal.Process
         whenCancellingProcess,
         -- * Awaiting Signal
         processAwait,
+        -- * Preemption
+        processPreemptionBegin,
+        processPreemptionEnd,
+        processPreemptionBeginning,
+        processPreemptionEnding,
         -- * Yield of Process
         processYield,
         -- * Process Timeout
@@ -75,12 +87,14 @@ module Simulation.Aivika.Internal.Process
         -- * Memoizing Process
         memoProcess,
         -- * Never Ending Process
-        neverProcess) where
+        neverProcess,
+        -- * Debugging
+        traceProcess) where
 
 import Data.Maybe
 import Data.IORef
 
-import Control.Exception (IOException, throw)
+import Control.Exception
 import Control.Monad
 import Control.Monad.Trans
 import Control.Applicative
@@ -91,15 +105,16 @@ import Simulation.Aivika.Internal.Simulation
 import Simulation.Aivika.Internal.Dynamics
 import Simulation.Aivika.Internal.Event
 import Simulation.Aivika.Internal.Cont
-import Simulation.Aivika.Internal.Signal
+import Simulation.Aivika.Signal
 
 -- | Represents a process identifier.
 data ProcessId = 
   ProcessId { processStarted :: IORef Bool,
               processReactCont     :: IORef (Maybe (ContParams ())), 
-              processCancelSource  :: ContCancellationSource,
+              processContId  :: ContId,
               processInterruptRef  :: IORef Bool, 
-              processInterruptCont :: IORef (Maybe (ContParams ())), 
+              processInterruptCont :: IORef (Maybe (ContParams ())),
+              processInterruptTime :: IORef Double,
               processInterruptVersion :: IORef Int }
 
 -- | Specifies a discontinuous process that can suspend at any time
@@ -126,12 +141,16 @@ holdProcess dt =
   Process $ \pid ->
   Cont $ \c ->
   Event $ \p ->
-  do let x = processInterruptCont pid
+  do when (dt < 0) $
+       error "Time period dt < 0: holdProcess"
+     let x = processInterruptCont pid
+         t = pointTime p + dt
      writeIORef x $ Just c
      writeIORef (processInterruptRef pid) False
+     writeIORef (processInterruptTime pid) t
      v <- readIORef (processInterruptVersion pid)
      invokeEvent p $
-       enqueueEvent (pointTime p + dt) $
+       enqueueEvent t $
        Event $ \p ->
        do v' <- readIORef (processInterruptVersion pid)
           when (v == v') $ 
@@ -159,6 +178,37 @@ processInterrupted pid =
   Event $ \p ->
   readIORef (processInterruptRef pid)
 
+-- | Define a reaction when the process with the specified identifier is preempted.
+processPreempted :: ProcessId -> Event ()
+processPreempted pid =
+  Event $ \p ->
+  do let x = processInterruptCont pid
+     a <- readIORef x
+     case a of
+       Just c ->
+         do writeIORef x Nothing
+            writeIORef (processInterruptRef pid) True
+            modifyIORef (processInterruptVersion pid) $ (+) 1
+            t <- readIORef (processInterruptTime pid)
+            let dt = t - pointTime p
+                c' = substituteCont c $ \a ->
+                  Event $ \p ->
+                  invokeEvent p $
+                  invokeCont c $
+                  invokeProcess pid $
+                  holdProcess dt
+            invokeEvent p $
+              reenterCont c' ()
+       Nothing ->
+         do let x = processReactCont pid
+            a <- readIORef x
+            case a of
+              Nothing ->
+                return ()
+              Just c ->
+                do let c' = substituteCont c $ reenterCont c
+                   writeIORef x $ Just c'
+
 -- | Passivate the process.
 passivateProcess :: Process ()
 passivateProcess =
@@ -202,11 +252,20 @@ processIdPrepare pid =
             "Another process with the specified identifier " ++
             "has been started already: processIdPrepare"
        else writeIORef (processStarted pid) True
-     let signal = processCancelling pid
+     let signal = contSignal $ processContId pid
      invokeEvent p $
-       handleSignal_ signal $ \_ ->
-       do interruptProcess pid
-          reactivateProcess pid
+       handleSignal_ signal $ \e ->
+       Event $ \p ->
+       case e of
+         ContCancellationInitiating ->
+           do z <- contCancellationActivated $ processContId pid
+              when z $
+                do invokeEvent p $ interruptProcess pid
+                   invokeEvent p $ reactivateProcess pid
+         ContPreemptionBeginning ->
+           invokeEvent p $ processPreempted pid
+         ContPreemptionEnding ->
+           return ()
 
 -- | Run immediately the process. A new 'ProcessId' identifier will be
 -- assigned to the process.
@@ -227,7 +286,7 @@ runProcess p =
 runProcessUsingId :: ProcessId -> Process () -> Event ()
 runProcessUsingId pid p =
   do processIdPrepare pid
-     runCont m cont econt ccont (processCancelSource pid) False
+     runCont m cont econt ccont (processContId pid) False
        where cont  = return
              econt = throwEvent
              ccont = return
@@ -277,36 +336,39 @@ newProcessId :: Simulation ProcessId
 newProcessId =
   do x <- liftIO $ newIORef Nothing
      y <- liftIO $ newIORef False
-     c <- newContCancellationSource
+     c <- newContId
      i <- liftIO $ newIORef False
      z <- liftIO $ newIORef Nothing
+     t <- liftIO $ newIORef 0
      v <- liftIO $ newIORef 0
      return ProcessId { processStarted = y,
                         processReactCont     = x, 
-                        processCancelSource  = c, 
+                        processContId  = c, 
                         processInterruptRef  = i,
-                        processInterruptCont = z, 
+                        processInterruptCont = z,
+                        processInterruptTime = t,
                         processInterruptVersion = v }
 
 -- | Cancel a process with the specified identifier, interrupting it if needed.
 cancelProcessWithId :: ProcessId -> Event ()
-cancelProcessWithId pid = contCancellationInitiate (processCancelSource pid)
+cancelProcessWithId pid = contCancellationInitiate (processContId pid)
 
 -- | The process cancels itself.
 cancelProcess :: Process a
 cancelProcess =
   do pid <- processId
      liftEvent $ cancelProcessWithId pid
-     throwProcess $ error "The process must be cancelled already: cancelProcess."
+     throwProcess $
+       (error "The process must be cancelled already: cancelProcess." :: SomeException)
 
 -- | Test whether the process with the specified identifier was cancelled.
 processCancelled :: ProcessId -> Event Bool
-processCancelled pid = contCancellationInitiated (processCancelSource pid)
+processCancelled pid = contCancellationInitiated (processContId pid)
 
 -- | Return a signal that notifies about cancelling the process with 
 -- the specified identifier.
 processCancelling :: ProcessId -> Signal ()
-processCancelling pid = contCancellationInitiating (processCancelSource pid)
+processCancelling pid = contCancellationInitiating (processContId pid)
 
 -- | Register a handler that will be invoked in case of cancelling the current process.
 whenCancellingProcess :: Event () -> Process ()
@@ -315,6 +377,22 @@ whenCancellingProcess h =
   liftEvent $
   handleSignal_ (processCancelling pid) $ \() -> h
 
+-- | Preempt a process with the specified identifier.
+processPreemptionBegin :: ProcessId -> Event ()
+processPreemptionBegin pid = contPreemptionBegin (processContId pid)
+
+-- | Proceed with the process with the specified identifier after it was preempted with help of 'preemptProcessBegin'.
+processPreemptionEnd :: ProcessId -> Event ()
+processPreemptionEnd pid = contPreemptionEnd (processContId pid)
+
+-- | Return a signal when the process is preempted.
+processPreemptionBeginning :: ProcessId -> Signal ()
+processPreemptionBeginning pid = contPreemptionBeginning (processContId pid)
+
+-- | Return a signal when the process is proceeded after it was preempted earlier.
+processPreemptionEnding :: ProcessId -> Signal ()
+processPreemptionEnding pid = contPreemptionEnding (processContId pid)
+
 instance Eq ProcessId where
   x == y = processReactCont x == processReactCont y    -- for the references are unique
 
@@ -377,7 +455,7 @@ liftIOP :: IO a -> Process a
 liftIOP m = Process $ \pid -> liftIO m
 
 -- | Exception handling within 'Process' computations.
-catchProcess :: Process a -> (IOException -> Process a) -> Process a
+catchProcess :: Exception e => Process a -> (e -> Process a) -> Process a
 catchProcess (Process m) h =
   Process $ \pid ->
   catchCont (m pid) $ \e ->
@@ -390,11 +468,13 @@ finallyProcess (Process m) (Process m') =
   finallyCont (m pid) (m' pid)
 
 -- | Throw the exception with the further exception handling.
--- By some reasons, the standard 'throw' function per se is not handled 
--- properly within 'Process' computations, although it will be still 
--- handled if it will be hidden under the 'liftIO' function. The problem 
--- arises namely with the @throw@ function, not 'IO' computations.
-throwProcess :: IOException -> Process a
+--
+-- By some reason, an exception raised with help of the standard 'throw' function
+-- is not handled properly within 'Process' computation, altough it will be still handled 
+-- if it will be wrapped in the 'IO' monad. Therefore, you should use specialised
+-- functions like the stated one that use the 'throw' function but within the 'IO' computation,
+-- which allows already handling the exception.
+throwProcess :: Exception e => e -> Process a
 throwProcess = liftIO . throw
 
 -- | Execute the specified computations in parallel within
@@ -421,7 +501,7 @@ processParallelUsingIds xs =
   do liftEvent $ processParallelPrepare xs
      contParallel $
        flip map xs $ \(pid, m) ->
-       (invokeProcess pid m, processCancelSource pid)
+       (invokeProcess pid m, processContId pid)
 
 -- | Like 'processParallel' but ignores the result.
 processParallel_ :: [Process a] -> Process ()
@@ -435,7 +515,7 @@ processParallelUsingIds_ xs =
   do liftEvent $ processParallelPrepare xs
      contParallel_ $
        flip map xs $ \(pid, m) ->
-       (invokeProcess pid m, processCancelSource pid)
+       (invokeProcess pid m, processContId pid)
 
 -- | Create the new process identifiers.
 processParallelCreateIds :: [Process a] -> Simulation [(ProcessId, Process a)]
@@ -461,22 +541,33 @@ processUsingId :: ProcessId -> Process a -> Process a
 processUsingId pid x =
   Process $ \pid' ->
   do liftEvent $ processIdPrepare pid
-     rerunCont (invokeProcess pid x) (processCancelSource pid)
+     rerunCont (invokeProcess pid x) (processContId pid)
+
+-- | Spawn the child process. In case of cancelling one of the processes,
+-- other process will be cancelled too.
+spawnProcess :: Process () -> Process ()
+spawnProcess = spawnProcessWith CancelTogether
+
+-- | Spawn the child process with the specified process identifier.
+-- In case of cancelling one of the processes, other process will
+-- be cancelled too.
+spawnProcessUsingId :: ProcessId -> Process () -> Process ()
+spawnProcessUsingId = spawnProcessUsingIdWith CancelTogether
 
 -- | Spawn the child process specifying how the child and parent processes
 -- should be cancelled in case of need.
-spawnProcess :: ContCancellation -> Process () -> Process ()
-spawnProcess cancellation x =
+spawnProcessWith :: ContCancellation -> Process () -> Process ()
+spawnProcessWith cancellation x =
   do pid <- liftSimulation newProcessId
-     spawnProcessUsingId cancellation pid x
+     spawnProcessUsingIdWith cancellation pid x
 
 -- | Spawn the child process specifying how the child and parent processes
 -- should be cancelled in case of need.
-spawnProcessUsingId :: ContCancellation -> ProcessId -> Process () -> Process ()
-spawnProcessUsingId cancellation pid x =
+spawnProcessUsingIdWith :: ContCancellation -> ProcessId -> Process () -> Process ()
+spawnProcessUsingIdWith cancellation pid x =
   Process $ \pid' ->
   do liftEvent $ processIdPrepare pid
-     spawnCont cancellation (invokeProcess pid x) (processCancelSource pid)
+     spawnCont cancellation (invokeProcess pid x) (processContId pid)
 
 -- | Await the signal.
 processAwait :: Signal a -> Process a
@@ -581,12 +672,12 @@ timeoutProcessUsingId :: Double -> ProcessId -> Process a -> Process (Maybe a)
 timeoutProcessUsingId timeout pid p =
   do s <- liftSimulation newSignalSource
      timeoutPid <- liftSimulation newProcessId
-     spawnProcessUsingId CancelChildAfterParent timeoutPid $
+     spawnProcessUsingIdWith CancelChildAfterParent timeoutPid $
        finallyProcess
        (holdProcess timeout)
        (liftEvent $
         cancelProcessWithId pid)
-     spawnProcessUsingId CancelChildAfterParent pid $
+     spawnProcessUsingIdWith CancelChildAfterParent pid $
        do r <- liftIO $ newIORef Nothing
           finallyProcess
             (catchProcess
@@ -601,7 +692,7 @@ timeoutProcessUsingId timeout pid p =
      case x of
        Nothing -> return Nothing
        Just (Right a) -> return (Just a)
-       Just (Left e) -> throwProcess e
+       Just (Left (SomeException e)) -> throwProcess e
 
 -- | Yield to allow other 'Process' and 'Event' computations to run
 -- at the current simulation time point.
@@ -625,3 +716,10 @@ neverProcess =
   let signal = processCancelling pid
   in handleSignal_ signal $ \_ ->
      resumeCont c $ error "It must never be computed: neverProcess"
+
+-- | Show the debug message with the current simulation time.
+traceProcess :: String -> Process a -> Process a
+traceProcess message m =
+  Process $ \pid ->
+  traceCont message $
+  invokeProcess pid m
