@@ -1,7 +1,7 @@
 
 -- |
 -- Module     : Simulation.Aivika.Processor
--- Copyright  : Copyright (c) 2009-2013, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2015, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
@@ -16,6 +16,7 @@ module Simulation.Aivika.Processor
         emptyProcessor,
         arrProcessor,
         accumProcessor,
+        withinProcessor,
         -- * Specifying Identifier
         processorUsingId,
         -- * Prefetch and Delay Processors
@@ -39,9 +40,15 @@ module Simulation.Aivika.Processor
         processorPrioritisingInputOutputParallel,
         -- * Arrival Processor
         arrivalProcessor,
+        -- * Utilities
+        joinProcessor,
+        -- * Failover
+        failoverProcessor,
         -- * Integrating with Signals
         signalProcessor,
-        processorSignaling) where
+        processorSignaling,
+        -- * Debugging
+        traceProcessor) where
 
 import qualified Control.Category as C
 import Control.Arrow
@@ -141,6 +148,13 @@ accumProcessor f acc =
          (acc', b) <- f acc a
          return (b, Cons $ loop xs' acc') 
 
+-- | Involve the computation with side effect when processing a stream of data.
+withinProcessor :: Process () -> Processor a a
+withinProcessor m =
+  Processor $
+  mapStreamM $ \a ->
+  do { m; return a }
+
 -- | Create a processor that will use the specified process identifier.
 -- It can be useful to refer to the underlying 'Process' computation which
 -- can be passivated, interrupted, canceled and so on. See also the
@@ -155,8 +169,8 @@ processorUsingId pid (Processor f) =
 -- If you don't know what the enqueue strategies to apply, then
 -- you will probably need 'FCFS' for the both parameters, or
 -- function 'processorParallel' that does namely this.
-processorQueuedParallel :: (EnqueueStrategy si qi,
-                            EnqueueStrategy so qo)
+processorQueuedParallel :: (EnqueueStrategy si,
+                            EnqueueStrategy so)
                            => si
                            -- ^ the strategy applied for enqueuing the input data
                            -> so
@@ -176,8 +190,8 @@ processorQueuedParallel si so ps =
      runStream output
 
 -- | Launches the specified processors in parallel using priorities for combining the output.
-processorPrioritisingOutputParallel :: (EnqueueStrategy si qi,
-                                        PriorityQueueStrategy so qo po)
+processorPrioritisingOutputParallel :: (EnqueueStrategy si,
+                                        PriorityQueueStrategy so po)
                                        => si
                                        -- ^ the strategy applied for enqueuing the input data
                                        -> so
@@ -197,8 +211,8 @@ processorPrioritisingOutputParallel si so ps =
      runStream output
 
 -- | Launches the specified processors in parallel using priorities for consuming the intput.
-processorPrioritisingInputParallel :: (PriorityQueueStrategy si qi pi,
-                                       EnqueueStrategy so qo)
+processorPrioritisingInputParallel :: (PriorityQueueStrategy si pi,
+                                       EnqueueStrategy so)
                                       => si
                                       -- ^ the strategy applied for enqueuing the input data
                                       -> so
@@ -219,8 +233,8 @@ processorPrioritisingInputParallel si so ps =
 
 -- | Launches the specified processors in parallel using priorities for consuming
 -- the input and combining the output.
-processorPrioritisingInputOutputParallel :: (PriorityQueueStrategy si qi pi,
-                                             PriorityQueueStrategy so qo po)
+processorPrioritisingInputOutputParallel :: (PriorityQueueStrategy si pi,
+                                             PriorityQueueStrategy so po)
                                             => si
                                             -- ^ the strategy applied for enqueuing the input data
                                             -> so
@@ -264,7 +278,7 @@ bufferProcessor :: (Stream a -> Process ())
 bufferProcessor consume output =
   Processor $ \xs ->
   Cons $
-  do spawnProcess CancelTogether (consume xs)
+  do spawnProcess (consume xs)
      runStream output
 
 -- | Like 'bufferProcessor' but allows creating a loop when some items
@@ -289,7 +303,7 @@ bufferProcessorLoop consume preoutput cond body =
        liftSimulation $
        partitionEitherStream $
        runProcessor cond preoutput
-     spawnProcess CancelTogether 
+     spawnProcess 
        (consume xs $ runProcessor body reverted)
      runStream output
 
@@ -396,9 +410,9 @@ queueProcessorLoopParallel :: (a -> Process ())
 queueProcessorLoopParallel enqueue dequeue =
   bufferProcessorLoop
   (\bs cs ->
-    do spawnProcess CancelTogether $
+    do spawnProcess $
          consumeStream enqueue bs
-       spawnProcess CancelTogether $
+       spawnProcess $
          consumeStream enqueue cs)
   (repeatProcess dequeue)
 
@@ -456,3 +470,27 @@ arrivalProcessor = Processor arrivalStream
 -- | A processor that delays the input stream by one step using the specified initial value.
 delayProcessor :: a -> Processor a a
 delayProcessor a0 = Processor $ delayStream a0
+
+-- | Removes one level of the computation, projecting its bound processor into the outer level.
+joinProcessor :: Process (Processor a b) -> Processor a b
+joinProcessor m =
+  Processor $ \xs ->
+  Cons $
+  do Processor f <- m
+     runStream $ f xs
+
+-- | Takes the next processor from the list after the current processor fails because of cancelling the underlying process.
+failoverProcessor :: [Processor a b] -> Processor a b
+failoverProcessor ps =
+  Processor $ \xs -> failoverStream [runProcessor p xs | p <- ps]
+
+-- | Show the debug messages with the current simulation time.
+traceProcessor :: Maybe String
+                  -- ^ the request message
+                  -> Maybe String
+                  -- ^ the response message
+                  -> Processor a b
+                  -- ^ a processor
+                  -> Processor a b
+traceProcessor request response (Processor f) =
+  Processor $ traceStream request response . f 
