@@ -1,11 +1,11 @@
 
 -- |
 -- Module     : Simulation.Aivika.Results.IO
--- Copyright  : Copyright (c) 2009-2014, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2015, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
--- Tested with: GHC 7.6.3
+-- Tested with: GHC 7.8.3
 --
 -- The module allows printing and converting the 'Simulation' 'Results' to a 'String'.
 --
@@ -107,24 +107,18 @@ hPrintResultSourceIndentedLabelled :: Handle
                                       -- ^ a localisation
                                       -> ResultSourcePrint
 hPrintResultSourceIndentedLabelled h indent label loc (ResultItemSource (ResultItem x)) =
-  case resultValueData (resultItemToStringValue x) of
-    Just m ->
-      do a <- m
-         let tab = replicate indent ' '
-         liftIO $
-           do hPutStr h tab
-              hPutStr h "-- "
-              hPutStr h (loc $ resultItemId x)
-              hPutStrLn h ""
-              hPutStr h tab
-              hPutStr h label
-              hPutStr h " = "
-              hPutStrLn h a
-              hPutStrLn h ""
-    _ ->
-      error $
-      "Expected to see a string value for variable " ++
-      (resultItemName x) ++ ": hPrintResultSourceIndentedLabelled"
+  do a <- resultValueData $ resultItemToStringValue x
+     let tab = replicate indent ' '
+     liftIO $
+       do hPutStr h tab
+          hPutStr h "-- "
+          hPutStr h (loc $ resultItemId x)
+          hPutStrLn h ""
+          hPutStr h tab
+          hPutStr h label
+          hPutStr h " = "
+          hPutStrLn h a
+          hPutStrLn h ""
 hPrintResultSourceIndentedLabelled h indent label loc (ResultVectorSource x) =
   do let tab = replicate indent ' '
      liftIO $
@@ -229,24 +223,18 @@ showResultSourceIndentedLabelled :: Int
                                    -- ^ a localisation
                                    -> ResultSourceShowS
 showResultSourceIndentedLabelled indent label loc (ResultItemSource (ResultItem x)) =
-  case resultValueData (resultItemToStringValue x) of
-    Just m ->
-      do a <- m
-         let tab = replicate indent ' '
-         return $
-           showString tab .
-           showString "-- " .
-           showString (loc $ resultItemId x) .
-           showString "\n" .
-           showString tab .
-           showString label .
-           showString " = " .
-           showString a .
-           showString "\n\n"
-    _ ->
-      error $
-      "Expected to see a string value for variable " ++
-      (resultItemName x) ++ ": showResultSourceIndentedLabelled"
+  do a <- resultValueData $ resultItemToStringValue x
+     let tab = replicate indent ' '
+     return $
+       showString tab .
+       showString "-- " .
+       showString (loc $ resultItemId x) .
+       showString "\n" .
+       showString tab .
+       showString label .
+       showString " = " .
+       showString a .
+       showString "\n\n"
 showResultSourceIndentedLabelled indent label loc (ResultVectorSource x) =
   do let tab = replicate indent ' '
          items = A.elems (resultVectorItems x)
