@@ -1,7 +1,7 @@
 
 -- |
 -- Module     : Simulation.Aivika.Server
--- Copyright  : Copyright (c) 2009-2013, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2015, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
@@ -13,6 +13,8 @@ module Simulation.Aivika.Server
         Server,
         newServer,
         newStateServer,
+        newPreemptibleServer,
+        newPreemptibleStateServer,
         -- * Processing
         serverProcessor,
         -- * Server Properties and Activities
@@ -21,12 +23,15 @@ module Simulation.Aivika.Server
         serverTotalInputWaitTime,
         serverTotalProcessingTime,
         serverTotalOutputWaitTime,
+        serverTotalPreemptionTime,
         serverInputWaitTime,
         serverProcessingTime,
         serverOutputWaitTime,
+        serverPreemptionTime,
         serverInputWaitFactor,
         serverProcessingFactor,
         serverOutputWaitFactor,
+        serverPreemptionFactor,
         -- * Summary
         serverSummary,
         -- * Derived Signals for Properties
@@ -38,20 +43,28 @@ module Simulation.Aivika.Server
         serverTotalProcessingTimeChanged_,
         serverTotalOutputWaitTimeChanged,
         serverTotalOutputWaitTimeChanged_,
+        serverTotalPreemptionTimeChanged,
+        serverTotalPreemptionTimeChanged_,
         serverInputWaitTimeChanged,
         serverInputWaitTimeChanged_,
         serverProcessingTimeChanged,
         serverProcessingTimeChanged_,
         serverOutputWaitTimeChanged,
         serverOutputWaitTimeChanged_,
+        serverPreemptionTimeChanged,
+        serverPreemptionTimeChanged_,
         serverInputWaitFactorChanged,
         serverInputWaitFactorChanged_,
         serverProcessingFactorChanged,
         serverProcessingFactorChanged_,
         serverOutputWaitFactorChanged,
         serverOutputWaitFactorChanged_,
+        serverPreemptionFactorChanged,
+        serverPreemptionFactorChanged_,
         -- * Basic Signals
         serverInputReceived,
+        serverTaskPreempting,
+        serverTaskReentering,
         serverTaskProcessed,
         serverOutputProvided,
         -- * Overall Signal
@@ -60,13 +73,14 @@ module Simulation.Aivika.Server
 import Data.IORef
 import Data.Monoid
 
+import Control.Monad
 import Control.Monad.Trans
 import Control.Arrow
 
 import Simulation.Aivika.Simulation
 import Simulation.Aivika.Dynamics
 import Simulation.Aivika.Internal.Event
-import Simulation.Aivika.Internal.Signal
+import Simulation.Aivika.Signal
 import Simulation.Aivika.Resource
 import Simulation.Aivika.Cont
 import Simulation.Aivika.Process
@@ -82,20 +96,30 @@ data Server s a b =
            -- ^ The current state of the server.
            serverProcess :: s -> a -> Process (s, b),
            -- ^ Provide @b@ by specified @a@.
+           serverProcessPreemptible :: Bool,
+           -- ^ Whether the process can be preempted.
            serverTotalInputWaitTimeRef :: IORef Double,
            -- ^ The counted total time spent in awating the input.
            serverTotalProcessingTimeRef :: IORef Double,
            -- ^ The counted total time spent to process the input and prepare the output.
            serverTotalOutputWaitTimeRef :: IORef Double,
            -- ^ The counted total time spent for delivering the output.
+           serverTotalPreemptionTimeRef :: IORef Double,
+           -- ^ The counted total time spent being preempted and waiting for the proceeding. 
            serverInputWaitTimeRef :: IORef (SamplingStats Double),
            -- ^ The statistics for the time spent in awaiting the input.
            serverProcessingTimeRef :: IORef (SamplingStats Double),
            -- ^ The statistics for the time spent to process the input and prepare the output.
            serverOutputWaitTimeRef :: IORef (SamplingStats Double),
            -- ^ The statistics for the time spent for delivering the output.
+           serverPreemptionTimeRef :: IORef (SamplingStats Double),
+           -- ^ The statistics for the time spent being preempted.
            serverInputReceivedSource :: SignalSource a,
            -- ^ A signal raised when the server recieves a new input to process.
+           serverTaskPreemptingSource :: SignalSource a,
+           -- ^ A signal raised when the task was preempted.
+           serverTaskReenteringSource :: SignalSource a,
+           -- ^ A signal raised when the task was proceeded after it had been preempted earlier.
            serverTaskProcessedSource :: SignalSource (a, b),
            -- ^ A signal raised when the input is processed and
            -- the output is prepared for deliverying.
@@ -104,48 +128,82 @@ data Server s a b =
          }
 
 -- | Create a new server that can provide output @b@ by input @a@.
--- Also it returns the corresponded processor that being applied
--- updates the server state.
+--
+-- By default, it is assumed that the server process cannot be preempted,
+-- because the handling of possible task preemption is rather costly
+-- operation.
 newServer :: (a -> Process b)
              -- ^ provide an output by the specified input
              -> Simulation (Server () a b)
-newServer provide =
-  flip newStateServer () $ \s a ->
-  do b <- provide a
-     return (s, b)
+newServer = newPreemptibleServer False
 
 -- | Create a new server that can provide output @b@ by input @a@
--- starting from state @s@. Also it returns the corresponded processor
--- that being applied updates the server state.
+-- starting from state @s@.
+--
+-- By default, it is assumed that the server process cannot be preempted,
+-- because the handling of possible task preemption is rather costly
+-- operation.
 newStateServer :: (s -> a -> Process (s, b))
                   -- ^ provide a new state and output by the specified 
                   -- old state and input
                   -> s
                   -- ^ the initial state
                   -> Simulation (Server s a b)
-newStateServer provide state =
+newStateServer = newPreemptibleStateServer False
+
+-- | Create a new preemptible server that can provide output @b@ by input @a@.
+newPreemptibleServer :: Bool
+                        -- ^ whether the server process can be preempted
+                        -> (a -> Process b)
+                        -- ^ provide an output by the specified input
+                        -> Simulation (Server () a b)
+newPreemptibleServer preemptible provide =
+  flip (newPreemptibleStateServer preemptible) () $ \s a ->
+  do b <- provide a
+     return (s, b)
+
+-- | Create a new preemptible server that can provide output @b@ by input @a@
+-- starting from state @s@.
+newPreemptibleStateServer :: Bool
+                             -- ^ whether the server process can be preempted
+                             -> (s -> a -> Process (s, b))
+                             -- ^ provide a new state and output by the specified 
+                             -- old state and input
+                             -> s
+                             -- ^ the initial state
+                             -> Simulation (Server s a b)
+newPreemptibleStateServer preemptible provide state =
   do r0 <- liftIO $ newIORef state
      r1 <- liftIO $ newIORef 0
      r2 <- liftIO $ newIORef 0
      r3 <- liftIO $ newIORef 0
-     r4 <- liftIO $ newIORef emptySamplingStats
+     r4 <- liftIO $ newIORef 0
      r5 <- liftIO $ newIORef emptySamplingStats
      r6 <- liftIO $ newIORef emptySamplingStats
+     r7 <- liftIO $ newIORef emptySamplingStats
+     r8 <- liftIO $ newIORef emptySamplingStats
      s1 <- newSignalSource
      s2 <- newSignalSource
      s3 <- newSignalSource
+     s4 <- newSignalSource
+     s5 <- newSignalSource
      let server = Server { serverInitState = state,
                            serverStateRef = r0,
                            serverProcess = provide,
+                           serverProcessPreemptible = preemptible,
                            serverTotalInputWaitTimeRef = r1,
                            serverTotalProcessingTimeRef = r2,
                            serverTotalOutputWaitTimeRef = r3,
-                           serverInputWaitTimeRef = r4,
-                           serverProcessingTimeRef = r5,
-                           serverOutputWaitTimeRef = r6,
+                           serverTotalPreemptionTimeRef = r4,
+                           serverInputWaitTimeRef = r5,
+                           serverProcessingTimeRef = r6,
+                           serverOutputWaitTimeRef = r7,
+                           serverPreemptionTimeRef = r8,
                            serverInputReceivedSource = s1,
-                           serverTaskProcessedSource = s2,
-                           serverOutputProvidedSource = s3 }
+                           serverTaskPreemptingSource = s2,
+                           serverTaskReenteringSource = s3,
+                           serverTaskProcessedSource = s4,
+                           serverOutputProvidedSource = s5 }
      return server
 
 -- | Return a processor for the specified server.
@@ -163,12 +221,12 @@ newStateServer provide state =
 -- in the chain. This is not always that thing you might need.
 --
 -- To model a sequence of the server processors working independently, you
--- should separate them with help of the 'prefetchProcessor' that plays a role
--- of a small one-place buffer in that case.
+-- should use the 'processorSeq' function which separates the processors with help of
+-- the 'prefetchProcessor' that plays a role of a small one-place buffer in that case.
 --
 -- The queue processors usually have the prefetching capabilities per se, where
 -- the items are already stored in the queue. Therefore, the server processor
--- should not be prefetched if it is connected directly with the queue processor.
+-- should not be prefetched if it is connected directly to the queue processor.
 serverProcessor :: Server s a b -> Processor a b
 serverProcessor server =
   Processor $ \xs -> loop (serverInitState server) Nothing xs
@@ -195,17 +253,54 @@ serverProcessor server =
                      addSamplingStats (t1 - t0)
               triggerSignal (serverInputReceivedSource server) a
          -- provide the service
-         (s', b) <- serverProcess server s a
+         (s', b, dt) <-
+           if serverProcessPreemptible server
+           then serverProcessPreempting server s a
+           else do (s', b) <- serverProcess server s a
+                   return (s', b, 0)
          t2 <- liftDynamics time
          liftEvent $
            do liftIO $
                 do writeIORef (serverStateRef server) $! s'
-                   modifyIORef' (serverTotalProcessingTimeRef server) (+ (t2 - t1))
+                   modifyIORef' (serverTotalProcessingTimeRef server) (+ (t2 - t1 - dt))
                    modifyIORef' (serverProcessingTimeRef server) $
-                     addSamplingStats (t2 - t1)
+                     addSamplingStats (t2 - t1 - dt)
               triggerSignal (serverTaskProcessedSource server) (a, b)
          return (b, loop s' (Just (t2, a, b)) xs')
 
+-- | Process the input with ability to handle a possible preemption.
+serverProcessPreempting :: Server s a b -> s -> a -> Process (s, b, Double)
+serverProcessPreempting server s a =
+  do pid <- processId
+     t1  <- liftDynamics time
+     rs  <- liftIO $ newIORef 0
+     r1  <- liftIO $ newIORef t1
+     h1  <- liftEvent $
+            handleSignal (processPreemptionBeginning pid) $ \() ->
+            do t1 <- liftDynamics time
+               liftIO $ writeIORef r1 t1
+               triggerSignal (serverTaskPreemptingSource server) a
+     h2  <- liftEvent $
+            handleSignal (processPreemptionEnding pid) $ \() ->
+            do t1 <- liftIO $ readIORef r1
+               t2 <- liftDynamics time
+               let dt = t2 - t1
+               liftIO $
+                 do modifyIORef' rs (+ dt)
+                    modifyIORef' (serverTotalPreemptionTimeRef server) (+ dt)
+                    modifyIORef' (serverPreemptionTimeRef server) $
+                      addSamplingStats dt
+               triggerSignal (serverTaskReenteringSource server) a 
+     let m1 =
+           do (s', b) <- serverProcess server s a
+              dt <- liftIO $ readIORef rs
+              return (s', b, dt)
+         m2 =
+           liftEvent $
+           do disposeEvent h1
+              disposeEvent h2
+     finallyProcess m1 m2
+
 -- | Return the current state of the server.
 --
 -- See also 'serverStateChanged' and 'serverStateChanged_'.
@@ -284,6 +379,27 @@ serverTotalOutputWaitTimeChanged_ :: Server s a b -> Signal ()
 serverTotalOutputWaitTimeChanged_ server =
   mapSignal (const ()) (serverOutputProvided server)
 
+-- | Return the counted total time spent by the server while it was preempted
+-- waiting for the further proceeding.
+--
+-- The value returned changes discretely and it is usually delayed relative
+-- to the current simulation time.
+--
+-- See also 'serverTotalPreemptionTimeChanged' and 'serverTotalPreemptionTimeChanged_'.
+serverTotalPreemptionTime :: Server s a b -> Event Double
+serverTotalPreemptionTime server =
+  Event $ \p -> readIORef (serverTotalPreemptionTimeRef server)
+  
+-- | Signal when the 'serverTotalPreemptionTime' property value has changed.
+serverTotalPreemptionTimeChanged :: Server s a b -> Signal Double
+serverTotalPreemptionTimeChanged server =
+  mapSignalM (const $ serverTotalPreemptionTime server) (serverTotalPreemptionTimeChanged_ server)
+  
+-- | Signal when the 'serverTotalPreemptionTime' property value has changed.
+serverTotalPreemptionTimeChanged_ :: Server s a b -> Signal ()
+serverTotalPreemptionTimeChanged_ server =
+  mapSignal (const ()) (serverTaskReentering server)
+
 -- | Return the statistics of the time when the server was locked while awaiting the input.
 --
 -- The value returned changes discretely and it is usually delayed relative
@@ -345,13 +461,34 @@ serverOutputWaitTimeChanged_ :: Server s a b -> Signal ()
 serverOutputWaitTimeChanged_ server =
   mapSignal (const ()) (serverOutputProvided server)
 
+-- | Return the statistics of the time spent by the server while it was preempted
+-- waiting for the further proceeding.
+--
+-- The value returned changes discretely and it is usually delayed relative
+-- to the current simulation time.
+--
+-- See also 'serverPreemptionTimeChanged' and 'serverPreemptionTimeChanged_'.
+serverPreemptionTime :: Server s a b -> Event (SamplingStats Double)
+serverPreemptionTime server =
+  Event $ \p -> readIORef (serverPreemptionTimeRef server)
+  
+-- | Signal when the 'serverPreemptionTime' property value has changed.
+serverPreemptionTimeChanged :: Server s a b -> Signal (SamplingStats Double)
+serverPreemptionTimeChanged server =
+  mapSignalM (const $ serverPreemptionTime server) (serverPreemptionTimeChanged_ server)
+  
+-- | Signal when the 'serverPreemptionTime' property value has changed.
+serverPreemptionTimeChanged_ :: Server s a b -> Signal ()
+serverPreemptionTimeChanged_ server =
+  mapSignal (const ()) (serverTaskReentering server)
+
 -- | It returns the factor changing from 0 to 1, which estimates how often
 -- the server was awaiting for the next input task.
 --
 -- This factor is calculated as
 --
 -- @
---   totalInputWaitTime \/ (totalInputWaitTime + totalProcessingTime + totalOutputWaitTime)
+--   totalInputWaitTime \/ (totalInputWaitTime + totalProcessingTime + totalOutputWaitTime + totalPreemptionTime)
 -- @
 --
 -- As before in this module, the value returned changes discretely and
@@ -364,7 +501,8 @@ serverInputWaitFactor server =
   do x1 <- readIORef (serverTotalInputWaitTimeRef server)
      x2 <- readIORef (serverTotalProcessingTimeRef server)
      x3 <- readIORef (serverTotalOutputWaitTimeRef server)
-     return (x1 / (x1 + x2 + x3))
+     x4 <- readIORef (serverTotalPreemptionTimeRef server)
+     return (x1 / (x1 + x2 + x3 + x4))
   
 -- | Signal when the 'serverInputWaitFactor' property value has changed.
 serverInputWaitFactorChanged :: Server s a b -> Signal Double
@@ -376,7 +514,8 @@ serverInputWaitFactorChanged_ :: Server s a b -> Signal ()
 serverInputWaitFactorChanged_ server =
   mapSignal (const ()) (serverInputReceived server) <>
   mapSignal (const ()) (serverTaskProcessed server) <>
-  mapSignal (const ()) (serverOutputProvided server)
+  mapSignal (const ()) (serverOutputProvided server) <>
+  mapSignal (const ()) (serverTaskReentering server)
 
 -- | It returns the factor changing from 0 to 1, which estimates how often
 -- the server was busy with direct processing its tasks.
@@ -384,7 +523,7 @@ serverInputWaitFactorChanged_ server =
 -- This factor is calculated as
 --
 -- @
---   totalProcessingTime \/ (totalInputWaitTime + totalProcessingTime + totalOutputWaitTime)
+--   totalProcessingTime \/ (totalInputWaitTime + totalProcessingTime + totalOutputWaitTime + totalPreemptionTime)
 -- @
 --
 -- As before in this module, the value returned changes discretely and
@@ -397,7 +536,8 @@ serverProcessingFactor server =
   do x1 <- readIORef (serverTotalInputWaitTimeRef server)
      x2 <- readIORef (serverTotalProcessingTimeRef server)
      x3 <- readIORef (serverTotalOutputWaitTimeRef server)
-     return (x2 / (x1 + x2 + x3))
+     x4 <- readIORef (serverTotalPreemptionTimeRef server)
+     return (x2 / (x1 + x2 + x3 + x4))
   
 -- | Signal when the 'serverProcessingFactor' property value has changed.
 serverProcessingFactorChanged :: Server s a b -> Signal Double
@@ -409,7 +549,8 @@ serverProcessingFactorChanged_ :: Server s a b -> Signal ()
 serverProcessingFactorChanged_ server =
   mapSignal (const ()) (serverInputReceived server) <>
   mapSignal (const ()) (serverTaskProcessed server) <>
-  mapSignal (const ()) (serverOutputProvided server)
+  mapSignal (const ()) (serverOutputProvided server) <>
+  mapSignal (const ()) (serverTaskReentering server)
 
 -- | It returns the factor changing from 0 to 1, which estimates how often
 -- the server was locked trying to deliver the output after the task is finished.
@@ -417,7 +558,7 @@ serverProcessingFactorChanged_ server =
 -- This factor is calculated as
 --
 -- @
---   totalOutputWaitTime \/ (totalInputWaitTime + totalProcessingTime + totalOutputWaitTime)
+--   totalOutputWaitTime \/ (totalInputWaitTime + totalProcessingTime + totalOutputWaitTime + totalPreemptionTime)
 -- @
 --
 -- As before in this module, the value returned changes discretely and
@@ -430,7 +571,8 @@ serverOutputWaitFactor server =
   do x1 <- readIORef (serverTotalInputWaitTimeRef server)
      x2 <- readIORef (serverTotalProcessingTimeRef server)
      x3 <- readIORef (serverTotalOutputWaitTimeRef server)
-     return (x3 / (x1 + x2 + x3))
+     x4 <- readIORef (serverTotalPreemptionTimeRef server)
+     return (x3 / (x1 + x2 + x3 + x4))
   
 -- | Signal when the 'serverOutputWaitFactor' property value has changed.
 serverOutputWaitFactorChanged :: Server s a b -> Signal Double
@@ -442,12 +584,56 @@ serverOutputWaitFactorChanged_ :: Server s a b -> Signal ()
 serverOutputWaitFactorChanged_ server =
   mapSignal (const ()) (serverInputReceived server) <>
   mapSignal (const ()) (serverTaskProcessed server) <>
-  mapSignal (const ()) (serverOutputProvided server)
+  mapSignal (const ()) (serverOutputProvided server) <>
+  mapSignal (const ()) (serverTaskReentering server)
+
+-- | It returns the factor changing from 0 to 1, which estimates how often
+-- the server was preempted waiting for the further proceeding.
+--
+-- This factor is calculated as
+--
+-- @
+--   totalPreemptionTime \/ (totalInputWaitTime + totalProcessingTime + totalOutputWaitTime + totalPreemptionTime)
+-- @
+--
+-- As before in this module, the value returned changes discretely and
+-- it is usually delayed relative to the current simulation time.
+--
+-- See also 'serverPreemptionFactorChanged' and 'serverPreemptionFactorChanged_'.
+serverPreemptionFactor :: Server s a b -> Event Double
+serverPreemptionFactor server =
+  Event $ \p ->
+  do x1 <- readIORef (serverTotalInputWaitTimeRef server)
+     x2 <- readIORef (serverTotalProcessingTimeRef server)
+     x3 <- readIORef (serverTotalOutputWaitTimeRef server)
+     x4 <- readIORef (serverTotalPreemptionTimeRef server)
+     return (x4 / (x1 + x2 + x3 + x4))
+  
+-- | Signal when the 'serverPreemptionFactor' property value has changed.
+serverPreemptionFactorChanged :: Server s a b -> Signal Double
+serverPreemptionFactorChanged server =
+  mapSignalM (const $ serverPreemptionFactor server) (serverPreemptionFactorChanged_ server)
+  
+-- | Signal when the 'serverPreemptionFactor' property value has changed.
+serverPreemptionFactorChanged_ :: Server s a b -> Signal ()
+serverPreemptionFactorChanged_ server =
+  mapSignal (const ()) (serverInputReceived server) <>
+  mapSignal (const ()) (serverTaskProcessed server) <>
+  mapSignal (const ()) (serverOutputProvided server) <>
+  mapSignal (const ()) (serverTaskReentering server)
 
 -- | Raised when the server receives a new input task.
 serverInputReceived :: Server s a b -> Signal a
 serverInputReceived = publishSignal . serverInputReceivedSource
 
+-- | Raised when the task processing by the server was preempted.
+serverTaskPreempting :: Server s a b -> Signal a
+serverTaskPreempting = publishSignal . serverTaskPreemptingSource
+
+-- | Raised when the task processing by the server was proceeded after it has been preempeted earlier.
+serverTaskReentering :: Server s a b -> Signal a
+serverTaskReentering = publishSignal . serverTaskReenteringSource
+
 -- | Raised when the server has just processed the task.
 serverTaskProcessed :: Server s a b -> Signal (a, b)
 serverTaskProcessed = publishSignal . serverTaskProcessedSource
@@ -461,7 +647,8 @@ serverChanged_ :: Server s a b -> Signal ()
 serverChanged_ server =
   mapSignal (const ()) (serverInputReceived server) <>
   mapSignal (const ()) (serverTaskProcessed server) <>
-  mapSignal (const ()) (serverOutputProvided server)
+  mapSignal (const ()) (serverOutputProvided server) <>
+  mapSignal (const ()) (serverTaskReentering server)
 
 -- | Return the summary for the server with desciption of its
 -- properties and activities using the specified indent.
@@ -471,12 +658,15 @@ serverSummary server indent =
   do tx1 <- readIORef (serverTotalInputWaitTimeRef server)
      tx2 <- readIORef (serverTotalProcessingTimeRef server)
      tx3 <- readIORef (serverTotalOutputWaitTimeRef server)
-     let xf1 = tx1 / (tx1 + tx2 + tx3)
-         xf2 = tx2 / (tx1 + tx2 + tx3)
-         xf3 = tx3 / (tx1 + tx2 + tx3)
+     tx4 <- readIORef (serverTotalPreemptionTimeRef server)
+     let xf1 = tx1 / (tx1 + tx2 + tx3 + tx4)
+         xf2 = tx2 / (tx1 + tx2 + tx3 + tx4)
+         xf3 = tx3 / (tx1 + tx2 + tx3 + tx4)
+         xf4 = tx4 / (tx1 + tx3 + tx3 + tx4)
      xs1 <- readIORef (serverInputWaitTimeRef server)
      xs2 <- readIORef (serverProcessingTimeRef server)
      xs3 <- readIORef (serverOutputWaitTimeRef server)
+     xs4 <- readIORef (serverPreemptionTimeRef server)
      let tab = replicate indent ' '
      return $
        showString tab .
@@ -489,6 +679,9 @@ serverSummary server indent =
        showString "total output wait time (locked while delivering the output) = " . shows tx3 .
        showString "\n\n" .
        showString tab .
+       showString "total preemption time = " . shows tx4 .
+       showString "\n" .
+       showString tab .
        showString "input wait factor (from 0 to 1) = " . shows xf1 .
        showString "\n" .
        showString tab .
@@ -498,6 +691,9 @@ serverSummary server indent =
        showString "output wait factor (from 0 to 1) = " . shows xf3 .
        showString "\n\n" .
        showString tab .
+       showString "output preemption factor (from 0 to 1) = " . shows xf4 .
+       showString "\n\n" .
+       showString tab .
        showString "input wait time (locked while awaiting the input):\n\n" .
        samplingStatsSummary xs1 (2 + indent) .
        showString "\n\n" .
@@ -507,4 +703,8 @@ serverSummary server indent =
        showString "\n\n" .
        showString tab .
        showString "output wait time (locked while delivering the output):\n\n" .
-       samplingStatsSummary xs3 (2 + indent)
+       samplingStatsSummary xs3 (2 + indent) .
+       showString "\n\n" .
+       showString tab .
+       showString "preemption time (waiting for the proceeding after preemption):\n\n" .
+       samplingStatsSummary xs4 (2 + indent)
