@@ -1,13 +1,14 @@
 
 -- |
 -- Module     : Simulation.Aivika.Simulation
--- Copyright  : Copyright (c) 2009-2013, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2015, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
 -- Tested with: GHC 7.8.3
 --
--- The module defines the 'Simulation' monad that represents a simulation run.
+-- The module defines the 'Simulation' monad that represents a computation within
+-- the simulation run.
 -- 
 module Simulation.Aivika.Simulation
        (-- * Simulation
@@ -20,6 +21,9 @@ module Simulation.Aivika.Simulation
         finallySimulation,
         throwSimulation,
         -- * Memoization
-        memoSimulation) where
+        memoSimulation,
+        -- * Exceptions
+        SimulationException(..),
+        SimulationAbort(..)) where
 
 import Simulation.Aivika.Internal.Simulation
