@@ -1,7 +1,7 @@
 
 -- |
 -- Module     : Simulation.Aivika.Statistics
--- Copyright  : Copyright (c) 2009-2013, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2015, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
@@ -27,7 +27,21 @@ module Simulation.Aivika.Statistics
         timingStatsDeviation,
         timingStatsSummary,
         returnTimingStats,
-        fromIntTimingStats) where 
+        fromIntTimingStats,
+        -- * Simple Counter
+        SamplingCounter(..),
+        emptySamplingCounter,
+        incSamplingCounter,
+        decSamplingCounter,
+        resetSamplingCounter,
+        returnSamplingCounter,
+        -- * Timing Counter
+        TimingCounter(..),
+        emptyTimingCounter,
+        incTimingCounter,
+        decTimingCounter,
+        resetTimingCounter,
+        returnTimingCounter) where
 
 import Data.Monoid
 
@@ -60,7 +74,7 @@ data SamplingStats a =
   deriving (Eq, Ord)
            
 -- | Specifies data type from which values we can gather the statistics.           
-class SamplingData a where           
+class Num a => SamplingData a where           
   
   -- | An empty statistics that has no samples.           
   emptySamplingStats :: SamplingStats a
@@ -230,6 +244,8 @@ data TimingStats a =
                 -- ^ Return the minimum value.
                 timingStatsMax       :: !a,
                 -- ^ Return the maximum value.
+                timingStatsLast      :: !a,
+                -- ^ Return the last value.
                 timingStatsMinTime   :: !Double,
                 -- ^ Return the time at which the minimum is attained.
                 timingStatsMaxTime   :: !Double,
@@ -245,7 +261,7 @@ data TimingStats a =
                 } deriving (Eq, Ord)
                            
 -- | Defines the data type from which values we can gather the timing statistics.
-class TimingData a where                           
+class Num a => TimingData a where                           
   
   -- | An empty statistics that has no samples.
   emptyTimingStats :: TimingStats a
@@ -265,6 +281,7 @@ instance TimingData Double where
     TimingStats { timingStatsCount     = 0,
                   timingStatsMin       = 1 / 0,
                   timingStatsMax       = (-1) / 0,
+                  timingStatsLast      = 0 / 0,
                   timingStatsMinTime   = 1 / 0,
                   timingStatsMaxTime   = (-1) / 0,
                   timingStatsStartTime = 1 / 0,
@@ -282,6 +299,7 @@ instance TimingData Int where
     TimingStats { timingStatsCount     = 0,
                   timingStatsMin       = maxBound,
                   timingStatsMax       = minBound,
+                  timingStatsLast      = 0,
                   timingStatsMinTime   = 1 / 0,
                   timingStatsMaxTime   = (-1) / 0,
                   timingStatsStartTime = 1 / 0,
@@ -300,6 +318,7 @@ addTimingStatsGeneric t a stats
   | count == 1 = TimingStats { timingStatsCount     = 1,
                                timingStatsMin       = a,
                                timingStatsMax       = a,
+                               timingStatsLast      = a,
                                timingStatsMinTime   = t,
                                timingStatsMaxTime   = t,
                                timingStatsStartTime = t,
@@ -309,6 +328,7 @@ addTimingStatsGeneric t a stats
   | otherwise  = TimingStats { timingStatsCount     = count,
                                timingStatsMin       = minX,
                                timingStatsMax       = maxX,
+                               timingStatsLast      = a,
                                timingStatsMinTime   = minT,
                                timingStatsMaxTime   = maxT,
                                timingStatsStartTime = t0,
@@ -326,11 +346,13 @@ addTimingStatsGeneric t a stats
                | otherwise = timingStatsMaxTime stats
           t0 = timingStatsStartTime stats
           t' = timingStatsLastTime stats
+          a' = timingStatsLast stats
           x  = convertToDouble a
+          x' = convertToDouble a'
           sumX'  = timingStatsSum stats
-          sumX   = sumX' + (t - t') * x
+          sumX   = sumX' + (t - t') * x'
           sumX2' = timingStatsSum2 stats
-          sumX2  = sumX2' + (t - t') * x * x
+          sumX2  = sumX2' + (t - t') * x' * x'
       
 timingStatsMeanGeneric :: ConvertableToDouble a => TimingStats a -> Double
 timingStatsMeanGeneric stats
@@ -370,8 +392,9 @@ returnTimingStats t a = addTimingStats t a emptyTimingStats
 -- | Convert the statistics from integer to double values.
 fromIntTimingStats :: TimingStats Int -> TimingStats Double
 fromIntTimingStats stats =
-  stats { timingStatsMin = fromIntegral $ timingStatsMin stats,
-          timingStatsMax = fromIntegral $ timingStatsMax stats }
+  stats { timingStatsMin  = fromIntegral $ timingStatsMin stats,
+          timingStatsMax  = fromIntegral $ timingStatsMax stats,
+          timingStatsLast = fromIntegral $ timingStatsLast stats }
 
 -- | Show the summary of the statistics.       
 showTimingStats :: (Show a, TimingData a) => TimingStats a -> ShowS
@@ -415,3 +438,83 @@ timingStatsSummary stats indent =
      showString "t in [" . shows (timingStatsStartTime stats) .
      showString ", " . shows (timingStatsLastTime stats) .
      showString "]"
+
+-- | A counter for which the statistics is collected too.
+data SamplingCounter a =
+  SamplingCounter { samplingCounterValue :: a,
+                    -- ^ The counter value.
+                    samplingCounterStats :: SamplingStats a
+                    -- ^ The counter statistics.
+                  } deriving (Eq, Ord, Show)
+
+-- | An empty counter.
+emptySamplingCounter :: SamplingData a => SamplingCounter a
+emptySamplingCounter =
+  SamplingCounter { samplingCounterValue = 0,
+                    samplingCounterStats = emptySamplingStats }
+
+-- | Increase the counter.
+incSamplingCounter :: SamplingData a => a -> SamplingCounter a -> SamplingCounter a
+incSamplingCounter a counter =
+  SamplingCounter { samplingCounterValue = a',
+                    samplingCounterStats = addSamplingStats a' (samplingCounterStats counter) }
+  where a' = samplingCounterValue counter + a
+
+-- | Decrease the counter.
+decSamplingCounter :: SamplingData a => a -> SamplingCounter a -> SamplingCounter a
+decSamplingCounter a counter =
+  SamplingCounter { samplingCounterValue = a',
+                    samplingCounterStats = addSamplingStats a' (samplingCounterStats counter) }
+  where a' = samplingCounterValue counter - a
+
+-- | Reset the counter.
+resetSamplingCounter :: SamplingData a => SamplingCounter a -> SamplingCounter a
+resetSamplingCounter counter =
+  SamplingCounter { samplingCounterValue = 0,
+                    samplingCounterStats = addSamplingStats 0 (samplingCounterStats counter) }
+
+-- | Create a counter with the specified initial value.
+returnSamplingCounter :: SamplingData  a => a -> SamplingCounter a
+returnSamplingCounter a =
+  SamplingCounter { samplingCounterValue = a,
+                    samplingCounterStats = returnSamplingStats a }
+
+-- | A counter for which the timing statistics is collected too.
+data TimingCounter a =
+  TimingCounter { timingCounterValue :: a,
+                  -- ^ The counter value.
+                  timingCounterStats :: TimingStats a
+                  -- ^ The counter statistics.
+                } deriving (Eq, Ord, Show)
+
+-- | An empty counter.
+emptyTimingCounter :: TimingData a => TimingCounter a
+emptyTimingCounter =
+  TimingCounter { timingCounterValue = 0,
+                  timingCounterStats = emptyTimingStats }
+
+-- | Increase the counter at the specified time.
+incTimingCounter :: TimingData a => Double -> a -> TimingCounter a -> TimingCounter a
+incTimingCounter t a counter =
+  TimingCounter { timingCounterValue = a',
+                  timingCounterStats = addTimingStats t a' (timingCounterStats counter) }
+  where a' = timingCounterValue counter + a
+
+-- | Decrease the counter at the specified time.
+decTimingCounter :: TimingData a => Double -> a -> TimingCounter a -> TimingCounter a
+decTimingCounter t a counter =
+  TimingCounter { timingCounterValue = a',
+                  timingCounterStats = addTimingStats t a' (timingCounterStats counter) }
+  where a' = timingCounterValue counter - a
+
+-- | Reset the counter at the specified time.
+resetTimingCounter :: TimingData a => Double -> TimingCounter a -> TimingCounter a
+resetTimingCounter t counter =
+  TimingCounter { timingCounterValue = 0,
+                  timingCounterStats = addTimingStats t 0 (timingCounterStats counter) }
+
+-- | Create a timing counter with the specified initial value at the given time.
+returnTimingCounter :: TimingData a => Double -> a -> TimingCounter a
+returnTimingCounter t a =
+  TimingCounter { timingCounterValue = a,
+                  timingCounterStats = returnTimingStats t a }
