@@ -1,7 +1,7 @@
 
 -- |
 -- Module     : Simulation.Aivika.Stream
--- Copyright  : Copyright (c) 2009-2013, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2015, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
@@ -46,11 +46,20 @@ module Simulation.Aivika.Stream
         repeatProcess,
         mapStream,
         mapStreamM,
-        apStreamDataFirst,
-        apStreamDataLater,
-        apStreamParallel,
+        apStream,
+        apStreamM,
         filterStream,
         filterStreamM,
+        takeStream,
+        takeStreamWhile,
+        takeStreamWhileM,
+        dropStream,
+        dropStreamWhile,
+        dropStreamWhileM,
+        singletonStream,
+        joinStream,
+        -- * Failover
+        failoverStream,
         -- * Integrating with Signals
         signalStream,
         streamSignal,
@@ -59,12 +68,15 @@ module Simulation.Aivika.Stream
         rightStream,
         replaceLeftStream,
         replaceRightStream,
-        partitionEitherStream) where
+        partitionEitherStream,
+        -- * Debugging
+        traceStream) where
 
 import Data.IORef
 import Data.Maybe
 import Data.Monoid
 
+import Control.Applicative
 import Control.Monad
 import Control.Monad.Trans
 
@@ -87,9 +99,19 @@ newtype Stream a = Cons { runStream :: Process (a, Stream a)
 
 instance Functor Stream where
   
-  fmap f (Cons s) = Cons y where
-    y = do ~(x, xs) <- s
-           return (f x, fmap f xs)
+  fmap = mapStream
+
+instance Applicative Stream where
+
+  pure a = let y = Cons (return (a, y)) in y
+  
+  (<*>) = apStream
+
+instance Alternative Stream where
+
+  empty = emptyStream
+
+  (<|>) = mergeStreams
 
 instance Monoid (Stream a) where
 
@@ -180,7 +202,9 @@ repeatProcess p = Cons y where
 
 -- | Map the stream according the specified function.
 mapStream :: (a -> b) -> Stream a -> Stream b
-mapStream = fmap
+mapStream f (Cons s) = Cons y where
+  y = do (a, xs) <- s
+         return (f a, mapStream f xs)
 
 -- | Compose the stream.
 mapStreamM :: (a -> Process b) -> Stream a -> Stream b
@@ -189,26 +213,20 @@ mapStreamM f (Cons s) = Cons y where
          b <- f a
          return (b, mapStreamM f xs)
 
--- | Transform the stream getting the transformation function after data have come.
-apStreamDataFirst :: Process (a -> b) -> Stream a -> Stream b
-apStreamDataFirst f (Cons s) = Cons y where
-  y = do ~(a, xs) <- s
-         g <- f
-         return (g a, apStreamDataFirst f xs)
+-- | Sequential application.
+apStream :: Stream (a -> b) -> Stream a -> Stream b
+apStream (Cons sf) (Cons sa) = Cons y where
+  y = do (f, sf') <- sf
+         (a, sa') <- sa
+         return (f a, apStream sf' sa')
 
--- | Transform the stream getting the transformation function before requesting for data.
-apStreamDataLater :: Process (a -> b) -> Stream a -> Stream b
-apStreamDataLater f (Cons s) = Cons y where
-  y = do g <- f
-         ~(a, xs) <- s
-         return (g a, apStreamDataLater f xs)
-
--- | Transform the stream trying to get the transformation function as soon as possible
--- at the same time when requesting for the next portion of data.
-apStreamParallel :: Process (a -> b) -> Stream a -> Stream b
-apStreamParallel f (Cons s) = Cons y where
-  y = do ~(g, (a, xs)) <- zipProcessParallel f s
-         return (g a, apStreamParallel f xs)
+-- | Sequential application.
+apStreamM :: Stream (a -> Process b) -> Stream a -> Stream b
+apStreamM (Cons sf) (Cons sa) = Cons y where
+  y = do (f, sf') <- sf
+         (a, sa') <- sa
+         x <- f a
+         return (x, apStreamM sf' sa')
 
 -- | Filter only those data values that satisfy to the specified predicate.
 filterStream :: (a -> Bool) -> Stream a -> Stream a
@@ -281,7 +299,7 @@ splitStream = splitStreamQueueing FCFS
 -- If you don't know what the strategy to apply, then you probably
 -- need the 'FCFS' strategy, or function 'splitStream' that
 -- does namely this.
-splitStreamQueueing :: EnqueueStrategy s q
+splitStreamQueueing :: EnqueueStrategy s
                        => s
                        -- ^ the strategy applied for enqueuing the output requests
                        -> Int
@@ -303,7 +321,7 @@ splitStreamQueueing s n x =
 
 -- | Split the input stream into a list of output streams
 -- using the specified priorities.
-splitStreamPrioritising :: PriorityQueueStrategy s q p
+splitStreamPrioritising :: PriorityQueueStrategy s p
                            => s
                            -- ^ the strategy applied for enqueuing the output requests
                            -> [Stream p]
@@ -335,7 +353,7 @@ concatStreams = concatQueuedStreams FCFS
 -- If you don't know what the strategy to apply, then you probably
 -- need the 'FCFS' strategy, or function 'concatStreams' that
 -- does namely this.
-concatQueuedStreams :: EnqueueStrategy s q
+concatQueuedStreams :: EnqueueStrategy s
                        => s
                        -- ^ the strategy applied for enqueuing the input data
                        -> [Stream a]
@@ -360,13 +378,13 @@ concatQueuedStreams s streams = Cons z where
                   liftIO $ writeIORef ref Nothing
                   releaseResource writing
                   return a
-         forM_ streams $ spawnProcess CancelTogether . writer
+         forM_ streams $ spawnProcess . writer
          a <- reader
          let xs = repeatProcess (releaseResource conting >> reader)
          return (a, xs)
 
 -- | Concatenate the input priority streams producing one output stream.
-concatPriorityStreams :: PriorityQueueStrategy s q p
+concatPriorityStreams :: PriorityQueueStrategy s p
                          => s
                          -- ^ the strategy applied for enqueuing the input data
                          -> [Stream (p, a)]
@@ -391,7 +409,7 @@ concatPriorityStreams s streams = Cons z where
                   liftIO $ writeIORef ref Nothing
                   releaseResource writing
                   return a
-         forM_ streams $ spawnProcess CancelTogether . writer
+         forM_ streams $ spawnProcess . writer
          a <- reader
          let xs = repeatProcess (releaseResource conting >> reader)
          return (a, xs)
@@ -405,7 +423,7 @@ mergeStreams = mergeQueuedStreams FCFS
 -- If you don't know what the strategy to apply, then you probably
 -- need the 'FCFS' strategy, or function 'mergeStreams' that
 -- does namely this.
-mergeQueuedStreams :: EnqueueStrategy s q
+mergeQueuedStreams :: EnqueueStrategy s
                       => s
                       -- ^ the strategy applied for enqueuing the input data
                       -> Stream a
@@ -417,7 +435,7 @@ mergeQueuedStreams :: EnqueueStrategy s q
 mergeQueuedStreams s x y = concatQueuedStreams s [x, y]
 
 -- | Merge two priority streams.
-mergePriorityStreams :: PriorityQueueStrategy s q p
+mergePriorityStreams :: PriorityQueueStrategy s p
                         => s
                         -- ^ the strategy applied for enqueuing the input data
                         -> Stream (p, a)
@@ -475,7 +493,7 @@ prefetchStream s = Cons z where
                   liftIO $ writeIORef ref Nothing
                   releaseResource writing
                   return a
-         spawnProcess CancelTogether $ writer s
+         spawnProcess $ writer s
          runStream $ repeatProcess reader
 
 -- | Return a stream of values triggered by the specified signal.
@@ -510,7 +528,7 @@ signalStream s =
 streamSignal :: Stream a -> Process (Signal a)
 streamSignal z =
   do s <- liftSimulation newSignalSource
-     spawnProcess CancelTogether $
+     spawnProcess $
        consumeStream (liftEvent . triggerSignal s) z
      return $ publishSignal s
 
@@ -532,3 +550,131 @@ arrivalStream s = Cons $ loop s Nothing where
 -- | Delay the stream by one step using the specified initial value.
 delayStream :: a -> Stream a -> Stream a
 delayStream a0 s = Cons $ return (a0, s)
+
+-- | Return a stream consisting of exactly one element and inifinite tail.
+singletonStream :: a -> Stream a
+singletonStream a = Cons $ return (a, emptyStream)
+
+-- | Removes one level of the computation, projecting its bound stream into the outer level.
+joinStream :: Process (Stream a) -> Stream a
+joinStream m = Cons $ m >>= runStream
+
+-- | Takes the next stream from the list after the current stream fails because of cancelling the underlying process.
+failoverStream :: [Stream a] -> Stream a
+failoverStream ps = Cons z where
+  z = do reading <- liftSimulation $ newResourceWithMaxCount FCFS 0 (Just 1)
+         writing <- liftSimulation $ newResourceWithMaxCount FCFS 0 (Just 1)
+         ref <- liftIO $ newIORef Nothing
+         pid <- processId
+         let writer p =
+               do requestResource writing
+                  pid' <- processId
+                  (a, xs) <-
+                    finallyProcess (runStream p) $
+                    liftEvent $
+                    do cancelled' <- processCancelled pid'
+                       when cancelled' $
+                         releaseResourceWithinEvent writing
+                  liftIO $ writeIORef ref (Just a)
+                  releaseResource reading
+                  writer xs
+             reader =
+               do releaseResource writing
+                  requestResource reading
+                  Just a <- liftIO $ readIORef ref
+                  liftIO $ writeIORef ref Nothing
+                  return a
+             loop [] = return ()
+             loop (p: ps) =
+               do pid' <- processId
+                  h' <- liftEvent $
+                        handleSignal (processCancelling pid) $ \() ->
+                        cancelProcessWithId pid'
+                  finallyProcess (writer p) $
+                    liftEvent $
+                    do disposeEvent h'
+                       cancelled <- processCancelled pid
+                       unless cancelled $
+                         do cancelled' <- processCancelled pid'
+                            unless cancelled' $
+                              error "Expected the sub-process to be cancelled: failoverStream"
+                            runProcess $ loop ps
+         liftEvent $ runProcess $ loop ps
+         runStream $ repeatProcess reader
+
+-- | Return the prefix of the stream of the specified length.
+takeStream :: Int -> Stream a -> Stream a
+takeStream n s
+  | n <= 0    = emptyStream
+  | otherwise =
+    Cons $
+    do (a, xs) <- runStream s
+       return (a, takeStream (n - 1) xs)
+
+-- | Return the longest prefix of the stream of elements that satisfy the predicate.
+takeStreamWhile :: (a -> Bool) -> Stream a -> Stream a
+takeStreamWhile p s =
+  Cons $
+  do (a, xs) <- runStream s
+     if p a
+       then return (a, takeStreamWhile p xs)
+       else neverProcess
+
+-- | Return the longest prefix of the stream of elements that satisfy the computation.
+takeStreamWhileM :: (a -> Process Bool) -> Stream a -> Stream a
+takeStreamWhileM p s =
+  Cons $
+  do (a, xs) <- runStream s
+     f <- p a
+     if f
+       then return (a, takeStreamWhileM p xs)
+       else neverProcess
+
+-- | Return the suffix of the stream after the specified first elements.
+dropStream :: Int -> Stream a -> Stream a
+dropStream n s
+  | n <= 0    = s
+  | otherwise =
+    Cons $
+    do (a, xs) <- runStream s
+       runStream $ dropStream (n - 1) xs
+
+-- | Return the suffix of the stream of elements remaining after 'takeStreamWhile'.
+dropStreamWhile :: (a -> Bool) -> Stream a -> Stream a
+dropStreamWhile p s =
+  Cons $
+  do (a, xs) <- runStream s
+     if p a
+       then runStream $ dropStreamWhile p xs
+       else return (a, xs)
+
+-- | Return the suffix of the stream of elements remaining after 'takeStreamWhileM'.
+dropStreamWhileM :: (a -> Process Bool) -> Stream a -> Stream a
+dropStreamWhileM p s =
+  Cons $
+  do (a, xs) <- runStream s
+     f <- p a
+     if f
+       then runStream $ dropStreamWhileM p xs
+       else return (a, xs)
+
+-- | Show the debug messages with the current simulation time.
+traceStream :: Maybe String
+               -- ^ the request message
+               -> Maybe String
+               -- ^ the response message
+               -> Stream a
+               -- ^ a stream
+               -> Stream a
+traceStream request response s = Cons $ loop s where
+  loop s = do (a, xs) <-
+                case request of
+                  Nothing -> runStream s
+                  Just message ->
+                    traceProcess message $
+                    runStream s
+              case response of
+                Nothing -> return (a, Cons $ loop xs)
+                Just message ->
+                  traceProcess message $
+                  return (a, Cons $ loop xs)
