@@ -3,7 +3,7 @@
 
 -- |
 -- Module     : Simulation.Aivika.SystemDynamics
--- Copyright  : Copyright (c) 2009-2013, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2015, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
@@ -25,6 +25,7 @@ module Simulation.Aivika.SystemDynamics
         ifDynamics,
         -- * Ordinary Differential Equations
         integ,
+        integEither,
         smoothI,
         smooth,
         smooth3I,
@@ -41,6 +42,7 @@ module Simulation.Aivika.SystemDynamics
         trend,
         -- * Difference Equations
         diffsum,
+        diffsumEither,
         -- * Table Functions
         lookupDynamics,
         lookupStepwiseDynamics,
@@ -65,7 +67,7 @@ import Simulation.Aivika.Internal.Specs
 import Simulation.Aivika.Internal.Parameter
 import Simulation.Aivika.Internal.Simulation
 import Simulation.Aivika.Internal.Dynamics
-import Simulation.Aivika.Dynamics.Interpolate
+import Simulation.Aivika.Dynamics.Extra
 import Simulation.Aivika.Unboxed
 import Simulation.Aivika.Table
 
@@ -263,6 +265,42 @@ integ diff i =
           RungeKutta4 -> return $ Dynamics $ integRK4 diff i y
       return y
 
+integEulerEither :: Dynamics (Either Double Double)
+                    -> Dynamics Double 
+                    -> Dynamics Double 
+                    -> Point -> IO Double
+integEulerEither (Dynamics f) (Dynamics i) (Dynamics y) p = 
+  case pointIteration p of
+    0 -> 
+      i p
+    n -> do 
+      let sc = pointSpecs p
+          ty = basicTime sc (n - 1) 0
+          py = p { pointTime = ty, pointIteration = n - 1, pointPhase = 0 }
+      b <- f py
+      case b of
+        Left v ->
+          return v
+        Right b -> do
+          a <- y py
+          let !v = a + spcDT (pointSpecs p) * b
+          return v
+
+-- | Like 'integ' but allows either setting a new 'Left' integral value,
+-- or integrating using the 'Right' derivative directly within computation.
+--
+-- This function always uses Euler's method.
+integEither :: Dynamics (Either Double Double)
+               -- ^ either set a new 'Left' integral value, or use a 'Right' derivative
+               -> Dynamics Double
+               -- ^ the initial value
+               -> Simulation (Dynamics Double)
+integEither diff i =
+  mdo y <- MU.memoDynamics z
+      z <- Simulation $ \r ->
+        return $ Dynamics $ integEulerEither diff i y
+      return y
+
 -- | Return the first order exponential smooth.
 --
 -- To create a loopback, you should use the recursive do-notation
@@ -492,6 +530,37 @@ diffsum (Dynamics diff) (Dynamics i) =
             return v
       return y
 
+-- | Like 'diffsum' but allows either setting a new 'Left' sum value, or adding the 'Right' difference.
+diffsumEither :: (Num a, Unboxed a)
+                 => Dynamics (Either a a)
+                 -- ^ either set the 'Left' value for the sum, or add the 'Right' difference to the sum
+                 -> Dynamics a
+                 -- ^ the initial value
+                 -> Simulation (Dynamics a)
+                 -- ^ the sum
+diffsumEither (Dynamics diff) (Dynamics i) =
+  mdo y <-
+        MU.memo0Dynamics $
+        Dynamics $ \p ->
+        case pointIteration p of
+          0 -> i p
+          n -> do 
+            let Dynamics m = y
+                sc = pointSpecs p
+                ty = basicTime sc (n - 1) 0
+                py = p { pointTime = ty, 
+                         pointIteration = n - 1, 
+                         pointPhase = 0 }
+            b <- diff py
+            case b of
+              Left v ->
+                return v
+              Right b -> do
+                a <- m py
+                let !v = a + b
+                return v
+      return y
+
 --
 -- Table Functions
 --
